#
# $Id$
#
import mipp.mda

class Metadata(mipp.mda.Metadata):
    token = ':'
    ignore_attributes = ('line_offset', 'first_pixel',
                         'coff', 'loff',
                         'image_data', 'boundaries')
if __name__ == '__main__':
    import sys
    print Metadata().read(sys.argv[1])



#
from mipp.xrit import sat
from mipp.xrit.mda import Metadata

# low level XRIT data readers.
from mipp.xrit._xrit import (read_prologue,
                             read_epilogue,
                             read_imagedata,
                             read_gts_message,
                             read_mpef_clm,
                             decompress,
                             list)



#
# $Id$ 
#
# Unpack binary data, all in network (big-endian) byte order.
#
import struct
from datetime import datetime, timedelta

def read_uint1(buf):
    return struct.unpack("!B", buf)[0]

def read_uint2(buf):
    return struct.unpack("!H", buf)[0]

def read_uint4(buf):
    return struct.unpack("!I", buf)[0]

def read_uint8(buf):
    v = struct.unpack("!2I", buf)
    return v[0]*pow(2L, 32) + v[1]

def read_int2(buf):
    return struct.unpack("!h", buf)[0]

def read_int4(buf):
    return struct.unpack("!i", buf)[0]

def read_float4(buf):
    return struct.unpack("!f", buf)[0]

def read_float8(buf):
    return struct.unpack("!d", buf)[0]


def read_cds_time(buf):
    days = read_uint2(buf[:2])
    msecs = read_uint4(buf[2:6])
    return datetime(1958, 1, 1) + timedelta(days=days, milliseconds=msecs)

def read_cds_expanded_time(buf):
    days = read_uint2(buf[:2])
    msecs = read_uint4(buf[2:6])
    usecs = read_uint2(buf[6:8])
    nsecs = read_uint2(buf[8:10])
    return datetime(1958, 1, 1) + timedelta(days=days, milliseconds=msecs, microseconds=(usecs+nsecs/1000.))

def read_cuc_time(buf, coarce, fine):
    ctime = 0
    ftime = 0
    for i in range(coarce - 1, -1, -1):
        ctime += ord(buf[coarce - i - 1]) * 2 ** (i * 8)
    for i in range(fine):
        ftime += ord(buf[coarce + i]) * 2 ** ((i + 1) * -8)
    return datetime(1958, 1, 1) + timedelta(seconds=ctime + ftime) 



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2012

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Read Electro L N1 HRIT files.
"""
# comments on the document:
# - data is in little endian
# - geometric processing's TagChGroup should not be there.
# - can't read the satellite name
# - explanation on types ?

import logging

logger = logging.getLogger(__name__)

from mipp.xrit import bin_reader as rbin
from mipp.xrit import Metadata
from StringIO import StringIO
from mipp.xrit import _xrit
import numpy as np


class _Calibrator(object):
    def __init__(self, hdr, channel_name):
        self.hdr = hdr
        self.vis = channel_name.startswith("00_")
        channels = ["00_6",
                    "00_7",
                    "00_9",
                    "03_8",
                    "06_4",
                    "08_0",
                    "08_7",
                    "09_7",
                    "10_7",
                    "11_9"]
        chnb = channels.index(channel_name)
        self.calibration_table = hdr["ImageCalibration"][chnb, :]
        
    def __call__(self, image, calibrate=1):
        if calibrate == 0:
            return image
        if calibrate == 1:
            res = np.ma.masked_less_equal(self.calibration_table[image], 0)
            if self.vis:
                return (res, "%")
            else:
                return (res, "K")
    

def read_proheader(fp):
    hdr = {}

    satstatus = [("TagType", "<u4"),
                 ("TagLength", "<u4"),
                 ("SatelliteID", "<u8"),
                 ("SatelliteName", "S256"),
                 ("NominalLongitude", "<f8"),
                 ("SatelliteCondition", "<u4"),
                 ("TimeOffset", "<f8")]
    
    satstatus = np.dtype(satstatus)
    hdr["SatelliteStatus"] =  np.fromstring(fp.read(satstatus.itemsize),
                                            dtype=satstatus,
                                            count=1)[0]


    imaq = [("TagType", "<u4"),
            ("TagLength", "<u4"),
            ("Status", "<u4"),
            ("StartDelay", "<i4"),
            ("Cel", "<f8")]

    imaq = np.dtype(imaq)
    hdr["ImageAcqusition"] =  np.fromstring(fp.read(imaq.itemsize*10),
                                            dtype=imaq,
                                            count=10)


    imcal = np.dtype("(10, 1024)<i4")

    hdr["ImageCalibration"] = np.fromstring(fp.read(imcal.itemsize),
                                            dtype=imcal,
                                            count=1)[0] / 1000.0
    
    return hdr

def read_epiheader(fp):

    hdr = {}

    rproc = [("TagType", "<u4"),
             ("TagLength", "<u4"),
             ("RPSummary",
              [("Impulse", "<u4"),
               ("IsStrNoiseCorrection", "<u4"),
               ("IsOptic", "<u4"),
               ("IsBrightnessAligment", "<u4")]),
             ("OpticCorrection",
              [("Degree", "<i4"),
               ("A", "<f8", (16, ))]),
             ("RPQuality",
              [("EffDinRange", "<f8"),
               ("EathDarkening", "<f8"),
               ("Zone", "<f8"),
               ("Impulse", "<f8"),
               ("Group", "<f8"),
               ("DefectCount", "<u4"),
               ("DefectProcent", "<f8"),
               ("S_Noise_DT_Preflight", "<f8"),
               ("S_Noise_DT_Bort", "<f8"),
               ("S_Noise_DT_Video", "<f8"),
               ("S_Noise_DT_1_5", "<f8"),
               ("CalibrStability", "<f8"),
               ("TemnSKO", "<f8", (2, )),
               ("StructSKO", "<f8", (2, )),
               ("Struct_1_5", "<f8"),
               ("Zone_1_ 5", "<f8"),
               ("RadDif", "<f8")])]

    rproc = np.dtype(rproc)
    hdr["RadiometricProcessing"] =  np.fromstring(fp.read(rproc.itemsize*10),
                                                  dtype=rproc,
                                                  count=10)
    gproc =  [("TagType", "<u4"),
              ("TagLength", "<u4"),
              ("TGeomNormInfo",
               [("IsExist", "<u4"),
                ("IsNorm", "<u4"),
                ("SubLon", "<f8"),
                ("TypeProjection", "<u4"),
                ("PixInfo", "<f8", (4, ))]),
              ("SatInfo",
               [("TISO",
                 [("T0", "<f8"),
                  ("dT", "<f8"),
                  ("ASb", "<f8"),
                  ("Evsk", "<f8", (3, 3, 4)),
                  ("ARx", "<f8", (4, )),
                  ("ARy", "<f8", (4, )),
                  ("ARz", "<f8", (4, )),
                  ("AVx", "<f8", (4, )),
                  ("AVy", "<f8", (4, )),
                  ("AVz", "<f8", (4, ))]),
                ("Type", "<i4")]),
              ("TimeProcessing", "<f8"),
              ("ApriorAccuracy", "<f8"),
              ("RelativeAccuracy", "<f8", (2, ))]

    gproc = np.dtype(gproc)
    hdr["GeometricProcessing"] =  np.fromstring(fp.read(gproc.itemsize*10),
                                                dtype=gproc,
                                                count=10)
    
    
    
    return hdr

def read_metadata(prologue, image_files, epilogue):
    """ Selected items from the Electro L N1 prolog file.
    """

    segment_size = 464 # number of lines in a segment

    hdr = {}

    fp = StringIO(prologue.data)
    phdr = read_proheader(fp)
    fp = StringIO(epilogue.data)
    ehdr = read_epiheader(fp)

    hdr.update(phdr)
    hdr.update(ehdr)

    im = _xrit.read_imagedata(image_files[0])

    md = Metadata()

    md.sublon = np.rad2deg(hdr["SatelliteStatus"]["NominalLongitude"])

    md.image_size = (im.structure.nc, im.structure.nc)

    md.channel = im.product_name[:4]
    md.satname = im.platform.lower()
    md.line_offset = 0
    md.data_type = im.structure.nb
    md.no_data_value = 0
    md.first_pixel = "north west"
    md.calibrate = _Calibrator(hdr, md.channel)

    segment_size = im.structure.nl
    md.loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
    md.coff = im.navigation.coff
    return md



import os
import sys
from datetime import datetime
import numpy
import unittest

import buildpath_to_syspath
import mipp
from mipp import xsar
from mipp.mda import _nice2cmp, mslice

datadir = (os.path.dirname(__file__) or '.') + '/data'
save_mda = False

try:
    # give the possibility to test other config files
    os.environ['PPP_CONFIG_DIR'] = os.environ['LOCAL_PPP_CONFIG_DIR']
except KeyError:
    os.environ['PPP_CONFIG_DIR'] = datadir
if not os.path.isdir(os.environ['PPP_CONFIG_DIR']):
    raise mipp.ConfigReaderError, "No config dir: '%s'"%os.environ['PPP_CONFIG_DIR']

txs1_file = datadir + '/TX01_SAR_SC_GEC_20110825T104705_20110825T104727_NSG_023264_8133_test.TSX.tar'
tsx1_sum = 3895.1342095

def make_image(mda, img, outdir='.'):
    if not os.environ.has_key('DEBUG'):
        return
    import Image as pil
    fname = outdir + '/' + mda.product_name + '.png'
    img = ((img - img.min()) * 255.0 /
           (img.max() - img.min()))
    if type(img) == numpy.ma.MaskedArray:
        img = img.filled(mda.no_data_value)
    img = pil.fromarray(numpy.array(img, numpy.uint8))
    img.save(fname)

def compare_mda(m1, m2):
    m1 = mslice(m1)
    m2 = mslice(m2)
    for m in (m1, m2):
        try:
            del m.tiff_params
        except AttributeError:
            pass
    k1 = sorted(m1.__dict__.keys())
    k2 = sorted(m2.__dict__.keys())
    if not k1 == k2:
        return False
    for k in k1:
        if not _nice2cmp(getattr(m1, k)) == _nice2cmp(getattr(m2, k)):
            return False           
    return True

class Test(unittest.TestCase):

    def test_tsx(self):
        mda, img = xsar.sat.load_file(txs1_file, 'sarx', calibrate=True)
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xsar.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='TSX metadata differ')
        self.assertTrue(img.shape == (512, 512), msg='TSX image reading failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, tsx1_sum, 3, msg='TSX image reading failed')

if __name__ == '__main__':
    save_mda = False
    unittest.main()



#
#
#
import os
import re
from ConfigParser import ConfigParser

import mipp

__all__ = ['read_config']

def read_config(satname, instrument=''):
    return _ConfigReader(satname, instrument)

class _ConfigReader(object):

    def __init__(self, satname, instrument=''):
        try:
            home = os.environ['PPP_CONFIG_DIR']
        except KeyError:
            raise mipp.ConfigReaderError(
                "PPP_CONFIG_DIR environment variable is not set")

        self.config_file = home + '/' + satname + '.cfg'
        if not os.path.isfile(self.config_file):
            raise mipp.ConfigReaderError(
                "unknown satellite: '%s' (no such file: '%s')"%
                (satname, self.config_file))
        self._config = ConfigParser()
        self._config.read(self.config_file)
        
        instruments = self.get('satellite')['instruments']
        if not instrument:
            if len(instruments) == 1:
                instrument = instruments[0]
            else:
                raise mipp.ConfigReaderError("please specify instrument")
        else:
            if instrument not in instruments: 
                raise mipp.ConfigReaderError("unknown instrument: '%s'"%
                                             instrument)
        self.instrument = instrument
        
        self._channels = self._channels2dict(instrument)

    def __call__(self, section):
        return self.get(section)

    def get(self, section):
        options = {}
        section = str(section) # allow get(1)
        if section != 'satellite' and not section.startswith(self.instrument):
            section = self.instrument + '-' + section
        for key, val in self._config.items(section, raw=True):
            options[key] = _eval(val)
        return options

    def get_channel(self, name):
        try:
            return self._channels[name]
        except KeyError:
            raise mipp.ConfigReaderError("unknown channel: '%s'"%name)

    @property
    def channels(self):
        return self._channels

    @property
    def channel_names(self):
        return sorted(self._channels.keys())

    def _channels2dict(self, instrument):
        rec = re.compile('^%s-\d+$'%instrument)
        channels = {}
        for sec in self._config.sections():
            if rec.findall(sec):
                chn = _Channel(self._config.items(sec, raw=True), raw=True)
                channels[chn.name] = chn
        return channels
    
class _Channel:
    def __init__(self, kvs, raw=False):
        self.name = None
        for key, val in kvs:
            if raw:
                val = _eval(val)
            setattr(self, key, val)
    def __str__(self):
        keys = sorted(self.__dict__.keys())
        text = ''
        for key in keys:
            if key[0] == '_':
                continue
            val = getattr(self, key)
            if key == 'resolution':
                val = "%.2f" % val
            elif key == 'frequency':
                val = "(%.2f, %.2f, %.2f)" % val
            text += key + ': ' + str(val) + ', '
        return text[:-2]    

def _eval(val):
    try:
        return eval(val)
    except:
        return str(val)

if __name__ == '__main__':
    import sys
    dname, fname = os.path.split(sys.argv[1])
    os.environ['PPP_CONFIG_DIR'] = dname
    cfg = read_config(os.path.splitext(fname)[0])
    for _name in ('satellite', 'level1', 'level2'):
        _sec = cfg(_name)
        print _name
        for _key in sorted(_sec.keys()):
            print '    ', _key + ':',  _sec[_key]
    for _name in cfg.channel_names:
        print cfg.get_channel(_name)



from StringIO import StringIO
import numpy as np

def dec10216(in_buffer):
    return _dec10216(in_buffer)

def hrpt_dec10216(in_buffer):
    #
    # It will handle the input data as raw HRPT data (level 0), which
    # originally is saved as packed 10 bit words width a record length
    # of (11090 - 2) (the last two words are ignored).
    # The AAPP software expect 10 bit right adjusted in 16 bit words
    # and a complete record length of 11090 words.
    #
    # !!! THIS ONE COULD BE FASTER !!!
    #
    HRPT_RECLEN = 11090
    fp = StringIO(dec10216(in_buffer))
    data = ''
    blob_size = (HRPT_RECLEN - 2)*2
    blob = fp.read(blob_size)
    while blob:
        if len(blob) == blob_size:
            blob += '\0\0\0\0'
        data += blob
        blob = fp.read(blob_size)
    return data

def _dec10216(inbuf):
    inbuf = np.fromstring(inbuf, dtype=np.uint8)
    arr10 = inbuf.astype(np.uint16)
    arr16 = np.zeros((len(arr10) / 5 * 4,), dtype=np.uint16)
    arr10_len = (len(arr16) * 5) / 4
    arr10 = arr10[:arr10_len] # adjust size
    """
    /*
     * pack 4 10-bit words in 5 bytes into 4 16-bit words
     * 
     * 0       1       2       3       4       5
     * 01234567890123456789012345678901234567890
     * 0         1         2         3         4
     */      
    ip = &in_buffer[i];
    op = &out_buffer[j];
    op[0] = ip[0]*4 + ip[1]/64;
    op[1] = (ip[1] & 0x3F)*16 + ip[2]/16;
    op[2] = (ip[2] & 0x0F)*64 + ip[3]/4;
    op[3] = (ip[3] & 0x03)*256 +ip[4];
    """
    arr16.flat[::4] = np.left_shift(arr10[::5], 2) + \
        np.right_shift((arr10[1::5]), 6)
    arr16.flat[1::4] = np.left_shift((arr10[1::5] & 63), 4) + \
        np.right_shift((arr10[2::5]), 4)
    arr16.flat[2::4] = np.left_shift(arr10[2::5] & 15, 6) + \
        np.right_shift((arr10[3::5]), 2)
    arr16.flat[3::4] = np.left_shift(arr10[3::5] & 3, 8) + \
        arr10[4::5]    
    return arr16.tostring()

if __name__ == '__main__':
    BLOB_SIZE = 10240 # has to be a multiply of 5
    import sys    
    try:
        if sys.argv[1] == 'hrpt':
            decoder = hrpt_dec10216
    except IndexError:
        decoder = dec10216
    blob = sys.stdin.read(BLOB_SIZE)
    while blob:
        sys.stdout.write(decoder(blob))
        blob = sys.stdin.read(BLOB_SIZE)
        



import sys
import os
import unittest
import cStringIO

import buildpath_to_syspath
print sys.path
import mipp.cfg

datadir = (os.path.dirname(__file__) or '.') + '/data'

class Test(unittest.TestCase):

    def test_config_parser(self):
        cfgfile = 'msg2'        
        os.environ['PPP_CONFIG_DIR'] = datadir
        c = mipp.cfg.read_config(cfgfile)
        fp = cStringIO.StringIO()
        for name in ('satellite', 'level1', 'level2'):
            h = c(name)
            print >>fp, name
            for k in sorted(h.keys()):
                print >>fp, '    ', k + ':',  h[k]
        print >>fp, mipp.cfg._Channel(c(1).items())
        print >>fp, mipp.cfg._Channel(c(2).items())
        print >>fp, mipp.cfg._Channel(c(3).items())
        for name in c.channel_names:
            print >>fp, c.get_channel(name)
        text1 = fp.getvalue().strip()
        fp.close()
        fp = open(datadir + '/' + cfgfile + '.cfg.out')
        text2 = fp.read().strip()
        fp.close()
        self.assertTrue(text1 == text2, msg='Reading %s.cfg failed'%cfgfile)

if __name__ == '__main__':
    unittest.main()

    



#
# WORK IN PROGRESS for Radarsat-2
#
import numpy
from datetime import datetime
from lxml import etree

from mipp.xsar import Metadata

class TiePoints(object):
    def __init__(self, image, geodedic):
        self.image = image
        self.geodedic = geodedic

def read_metadata(xml_file):
    metadata = Metadata()    

    # XML Namespace
    ns_rs2 = {'xsi': 'http://www.rsi.ca/rs2/prod/xml/schemas'}

    # Speciel decoders
    def dec_isoformat(rts):
        return datetime.strptime(rts, "%Y-%m-%dT%H:%M:%S.%fZ")
    def dec_orbit_number(rts):
        return int(rts[:5])

    attributes = {
        'product_id': ('xsi:productId', str),
        'satellite_name': ('xsi:sourceAttributes/xsi:satellite', str),
        'sensor_name': ('xsi:sourceAttributes/xsi:sensor', str),
        'beam_mode': ('xsi:sourceAttributes/xsi:beamModeMnemonic', str),
        'facility_id': ('xsi:sourceAttributes/xsi:inputDatasetFacilityId', str),
        'start_time': ('xsi:sourceAttributes/xsi:rawDataStartTime', dec_isoformat),
        'orbit_number': ('xsi:sourceAttributes/xsi:orbitAndAttitude/xsi:orbitInformation/xsi:orbitDataFile', dec_orbit_number),
        'product_format': ('xsi:imageAttributes/xsi:productFormat', str),
        'bits_per_sample': ('xsi:imageAttributes/xsi:rasterAttributes/xsi:bitsPerSample', int),
        'samples': ('xsi:imageAttributes/xsi:rasterAttributes/xsi:numberOfSamplesPerLine', int),
        'lines': ('xsi:imageAttributes/xsi:rasterAttributes/xsi:numberOfLines', int),
        'sample_spacing': ('xsi:imageAttributes/xsi:rasterAttributes/xsi:sampledPixelSpacing', float),
        'line_spacing': ('xsi:imageAttributes/xsi:rasterAttributes/xsi:sampledLineSpacing', float),
        'data_files': ('xsi:imageAttributes/xsi:fullResolutionImageData', str),
        'center_lat': ('centre_lat', str),
        'center_lon': ('centre_lon', str),
        'tie_point_lines': ('tie_point_lines', str),
        'tie_point_samples': ('tie_point_samples', str),
        'tie_point_line_jump': ('tie_point_line_jump', str),
        'tie_point_sample_jump': ('tie_point_sample_jump', str)}

    tree = etree.parse(xml_file)

    # Get some attributes
    for key, (att, dec) in attributes.items():
        if att.startswith('xsi'):
            rec = tree.xpath(att, namespaces=ns_rs2)
            if len(rec) > 1:
                val = tuple([dec(i.text) for i in rec])
            else:
                val = dec(rec[0].text)
            setattr(metadata, key, val)

    #
    # Get tiepoints
    #
    tiepoints_xpath = 'xsi:imageAttributes/xsi:geographicInformation/xsi:geolocationGrid/xsi:imageTiePoint'
    tiepoints_tree = tree.xpath(tiepoints_xpath, namespaces=ns_rs2)
    tiepoints_count = len(tiepoints_tree)

    pix_coordinates = numpy.zeros((tiepoints_count, 2))
    geo_coordinates = numpy.zeros((tiepoints_count, 2))
    counter = 0
    for elm in tiepoints_tree:
        if elm.tag.endswith('imageTiePoint'):
            pixel, line, lat, lon = None, None, None, None
            for i in elm.iter():
                if i.getparent().tag.endswith('imageCoordinate'):
                    if i.tag.endswith('pixel'):
                        pixel = float(i.text)
                    elif i.tag.endswith('line'):
                        line = float(i.text)
                elif i.getparent().tag.endswith('geodeticCoordinate'):
                    if i.tag.endswith('latitude'):
                        lat = float(i.text)
                    elif i.tag.endswith('longitude'):
                        lon = float(i.text)
            if None not in (pixel, line, lat, lon):
                pix_coordinates[counter] = [line, pixel]
                geo_coordinates[counter] = [lat, lon]
        counter += 1
        
    if counter > 0:
        setattr(metadata, 'tiepoints', TiePoints(pix_coordinates, geo_coordinates))

    return metadata

if __name__ == '__main__':
    import sys
    mda = read_metadata(sys.argv[1])
    print mda
    print mda.tiepoints.image
    print mda.tiepoints.geodedic



#
# $Id$ 
#

"""This module will read LRIT/HRIT headers. Format described in:
"LRIT/HRIT Global Specification"; CGMS 03; Issue 2.6; 12 August 1999
"MSG Ground Segment LRIT/HRIT Mission Specific Implementation"; EUM/MSG/SPE/057; Issue 6; 21 June 2006
"""

import sys
import os
from StringIO import StringIO 

import mipp
from mipp.xrit import bin_reader as rbin

__all__ = ['read_prologue',
           'read_epilogue',
           'read_imagedata',
           'read_gts_message',
           'read_mpef_clm',
           'decompress',
           'list']

def decompress(infile, outdir='.'):
    """Will decompress a XRIT data file and return the path to the decompressed file.
    It expect to find Eumetsat's xRITDecompress through the environment variable XRIT_DECOMPRESS_PATH 
    """
    from subprocess import Popen, PIPE
    cmd = os.environ.get('XRIT_DECOMPRESS_PATH', None)
    if not cmd:
        raise mipp.DecodeError("XRIT_DECOMPRESS_PATH is not defined (path to xRITDecompress")
        
    cwd = os.getcwd()
    os.chdir(outdir)
    p = Popen([cmd, infile], stdout=PIPE)
    stdout = StringIO(p.communicate()[0])
    status = p.returncode
    os.chdir(cwd)
    
    outfile = ''
    for line in stdout:
        try:
            k, v = [x.strip() for x in line.split(':', 1)]
        except ValueError:
            break
        if k == 'Decompressed file':
            outfile = v
            break
    
    if status != 0:
        raise mipp.DecodeError("xrit_decompress '%s', failed, status=%d"%(infile, status))
    if not outfile:
        raise mipp.DecodeError("xrit_decompress '%s', failed, no output file is generated"%infile)
    return outdir + '/' + outfile    
    
#-----------------------------------------------------------------------------
#
# XRIT header records
#
#-----------------------------------------------------------------------------
class PrimaryHeader(object):
    hdr_type = 0
    hdr_name = 'primary_header'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.file_type = rbin.read_uint1(fp.read(1))
        self.total_hdr_len = rbin.read_uint4(fp.read(4))
        self.data_field_len = rbin.read_uint8(fp.read(8))
        
    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, file_type:%d, total_hdr_len:%d, data_field_len:%d"%\
               (self.hdr_type, self.rec_len, self.file_type, self.total_hdr_len, self.data_field_len)

class ImageStructure(object):
    hdr_type = 1
    hdr_name = 'structure'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.nb = rbin.read_uint1(fp.read(1))
        self.nc = rbin.read_uint2(fp.read(2))
        self.nl = rbin.read_uint2(fp.read(2))
        self.compress_flag = rbin.read_uint1(fp.read(1))
        
    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, nb:%d, nc:%d, nl:%d, compress_flag:%d"%\
               (self.hdr_type, self.rec_len, self.nb, self.nc, self.nl, self.compress_flag)

class ImageNavigation(object):
    hdr_type = 2
    hdr_name = 'navigation'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.proj_name = fp.read(32).strip()
        self.cfac = rbin.read_int4(fp.read(4))
        self.lfac = rbin.read_int4(fp.read(4))
        self.coff = rbin.read_int4(fp.read(4))
        self.loff = rbin.read_int4(fp.read(4))
        i1 = self.proj_name.find('(')
        i2 = self.proj_name.find(')')
        if i1 != -1 and i2 != -1:
            self.ssp = float(self.proj_name[i1+1:i2])
        else:
            self.ssp = None
        
    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, proj_name:'%s', cfac:%d, lfac:%d. coff:%d, loff:%d"%\
               (self.hdr_type, self.rec_len, self.proj_name, self.cfac, self.lfac, self.coff, self.loff)

class ImageDataFunction(object):
    hdr_type = 3
    hdr_name = 'data_function'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.data_definition = _decode_data_definition(fp.read(self.rec_len-3))
        
    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, data_definition:'%s'"%\
               (self.hdr_type, self.rec_len, self.data_definition)

class AnnotationHeader(object):
    hdr_type = 4
    hdr_name = 'annotation'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.text = fp.read(self.rec_len-3).strip()
        a = [x.strip('_') for x in self.text.split('-')]
        self.xrit_channel_id = a[0]
        self.dissemination_id = int(a[1])
        self.dissemination_sc = a[2]
        self.platform = a[3]
        self.product_name = a[4]
        self.segment_name = a[5]
        self.time_stamp = mipp.strptime(a[6], "%Y%m%d%H%M")
        self.flags = a[7]
        self.segment_id = a[3] + '_' + a[4] + '_' + a[5] + '_' + self.time_stamp.strftime("%Y%m%d_%H%M")
        self.product_id = a[3] + '_' + a[4] + '_' + self.time_stamp.strftime("%Y%m%d_%H%M")

    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, text:%s"%\
               (self.hdr_type, self.rec_len, self.text)

class TimeStampRecord(object):
    hdr_type = 5
    hdr_name = 'time_stamp'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.cds_p_field = rbin.read_uint1(fp.read(1))
        self.time_stamp = rbin.read_cds_time(fp.read(6))

    def __str__(self):
        return  "hdr_type:%d, rec_len:%d, time_stamp:%s"%\
               (self.hdr_type, self.rec_len, str(self.time_stamp))

class SegmentIdentification(object):
    hdr_type = 128
    hdr_name = 'segment'    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        self.gp_sc_id = rbin.read_uint2(fp.read(2))
        self.spectral_channel_id = rbin.read_uint1(fp.read(1))
        self.seg_no = rbin.read_uint2(fp.read(2))
        self.planned_start_seg_no = rbin.read_uint2(fp.read(2))
        self.planned_end_seg_no = rbin.read_uint2(fp.read(2))
        self.data_field_repr = rbin.read_uint1(fp.read(1))

    def __str__(self):
        return  "hdr_type:%d, rec_len:%d gp_sc_id:%d, spectral_channel_id:%d, seg_no:%d, planned_start_seg_no:%d, planned_end_seg_no:%d, data_field_repr:%d"%\
               (self.hdr_type, self.rec_len, self.gp_sc_id, self.spectral_channel_id,\
                self.seg_no, self.planned_start_seg_no, self.planned_end_seg_no, self.data_field_repr)

class ImageSegmentLineQuality(object):
    hdr_type = 129
    hdr_name = 'image_quality'
    
    def __init__(self, fp):
        self.rec_len = rbin.read_uint2(fp.read(2))
        a = []
        nb = 3
        while nb < (self.rec_len):
            ln = rbin.read_int4(fp.read(4))
            stamp = rbin.read_cds_time(fp.read(6))
            lv = rbin.read_uint1(fp.read(1))
            lr = rbin.read_uint1(fp.read(1))
            lg = rbin.read_uint1(fp.read(1))
            a.append((ln, stamp, lv, lr, lg))
            #print ln, lv, lr, lg, stamp
            nb += 13            
        self.line_quality = a        

    def __str__(self):
        return  "hdr_type:%d, rec_len:%d"%\
               (self.hdr_type, self.rec_len)

class UnknownHeader(object):
    hdr_name = 'unknown'    
    def __init__(self, hdr_type, fp):
        self.hdr_type = hdr_type
        self.rec_len = rbin.read_uint2(fp.read(2))        
        self.data = fp.read(self.rec_len-3)
    def __str__(self):
        return  "hdr_type:%d, rec_len:%d"%\
               (self.hdr_type, self.rec_len)

def _decode_data_definition(buf):
    dd = dict()
    lines = [x.strip() for x in buf.strip().split('\r')]
    for a in lines:
        k, v = [x.strip() for x in a.split(':=')]
        if k[0] == '$':
            dd[k] = int(v)
        elif k[0] == '_':
            dd[k] = v
        elif k.isdigit():
            dd[int(k)] = float(v)
        else:
            raise mipp.DecodeError("could not decode data definition: '%s'"%a)
    return dd
    
header_map = {0: PrimaryHeader,
              1: ImageStructure,
              2: ImageNavigation,
              3: ImageDataFunction,
              4: AnnotationHeader,
              5: TimeStampRecord,
              128: SegmentIdentification,
              129: ImageSegmentLineQuality}
header_types = tuple(sorted(header_map.keys()))

def read_header(fp):
    hdr_type = rbin.read_uint1(fp.read(1))
    if hdr_type != 0:
        raise mipp.DecodeError("first header has to be a Primary Header, this one is of type %d"%hdr_type)
    phdr = PrimaryHeader(fp)
    yield phdr
    current_size = phdr.rec_len
    while current_size < phdr.total_hdr_len:
        hdr_type = rbin.read_uint1(fp.read(1))
        cls = header_map.get(hdr_type, None)
        if cls:
            hdr = cls(fp)
        else:
            hdr = UnknownHeader(hdr_type, fp)
        yield hdr
        current_size += hdr.rec_len

def read_headers(fp):
    return [h for h in read_header(fp)]

#-----------------------------------------------------------------------------
#
# File level
#
#-----------------------------------------------------------------------------
class Segment(object):
    def __init__(self, file_name):
        self.file_name = file_name
        fp = open(file_name)
        for h in read_header(fp):
            if h.hdr_type == 0:
                self.file_type = h.file_type
            elif h.hdr_type == 4:
                self.platform = h.platform
                self.product_name = h.product_name
                self.segment_name = h.segment_name
                self.time_stamp = h.time_stamp
                self.product_id = h.product_id
                self.segment_id = h.segment_id
            elif h.hdr_type == 5:
                self.production_time = h.time_stamp
            elif h.hdr_type in header_types:
                setattr(self, h.hdr_name, h)
        fp.close()
        try:
            self.is_compressed = bool(self.structure.compress_flag)
        except AttributeError:
            self.is_compressed = False
        # lazy reading of data
        self._blob = None

    @property
    def data(self):
        if not self._blob:
            fp = open(self.file_name)
            read_headers(fp)
            self._blob = fp.read()
            fp.close
        return self._blob

    def pprint(self):
        keys = self.__dict__.keys()
        keys.sort()
        for k in keys:
            if not k.startswith('_'):
                print k + ':', self.__dict__[k]

    def __str__(self):
        return self.segment_id

class ImageSegment(Segment):

    def __init__(self, file_name):
        Segment.__init__(self, file_name)
        self.bytes_per_line = (self.structure.nc*self.structure.nb)/8
        self.fp = None
    
    def readline(self, nlines=1):
        if not self.fp:
            self.fp = open(self.file_name)
            read_headers(self.fp)
        data = self.fp.read(self.bytes_per_line*nlines)
        if not data:
            raise mipp.DecodeError("could not read", self.bytes_per_line*nlines, "bytes")
        return data
    
    def close(self):
        if self.fp:
            self.fp.close()
            self.fp = None

def read_prologue(file_name):
    s = Segment(file_name)
    if s.file_type == 128:
        return s
    else:
        raise mipp.DecodeError("this is no 'prologue' file: '%s'"%file_name)

def read_epilogue(file_name):
    s = Segment(file_name)
    if s.file_type == 129:
        return s
    else:
        raise mipp.DecodeError("this is no 'epilogue' file: '%s'"%file_name)

def read_imagedata(file_name):
    s = Segment(file_name)
    if s.file_type == 0:
        return ImageSegment(file_name)
    else:
        raise mipp.DecodeError("this is no 'image data' file: '%s'"%file_name)
    
    
def read_gts_message(file_name):
    s = Segment(file_name)
    if s.file_type == 1:
        return s
    else:
        raise mipp.DecodeError("this is no 'GTS Message' file: '%s'"%file_name)
    
def read_mpef_clm(file_name):
    s = Segment(file_name)
    if s.file_type == 144:
        return s
    else:
        raise mipp.DecodeError("this is no 'MPEF cloud mask' file: '%s'"%file_name)
    
def list(file_name, dump_data=False):
    fname = 'xrit.dat'
    fp = open(file_name)
    for hdr in read_header(fp):
        print hdr
        if hdr.hdr_name == 'annotation':
            fname = hdr.segment_id
    data = fp.read()
    fp.close()
    if dump_data:
        print 'Writing', fname
        fp = open(fname, 'wb')
        fp.write(data)
        fp.close()
    
#-----------------------------------------------------------------------------
if __name__ == '__main__':
    args = sys.argv[1:]
    if len(args) > 1:
        if args[0] == '-d':
            dump_data = True
        filename = args[1]
    else:
        dump_data = False
        filename = args[0]
        
    list(filename, dump_data)



#
# $Id$ 
#
import numpy
import glob
import imp
import types
import re

import logging
logger = logging.getLogger('mipp')

import mipp
import mipp.cfg
from mipp.xrit import _xrit 
from mipp.xrit.loader import ImageLoader

__all__ = ['load_meteosat07',
           'load_meteosat09',
           'load_goes11',
           'load_goes12',
           'load_goes13',
           'load_mtsat1r',
           'load_mtsat2',
           'load_electrol',
           'load',
           'load_files']

CHECK_CONFIG_SUBLON = False

class SatelliteLoader(object):
    # Currently this one only works for geos satellites
    #
    # We will return an ImageLoader object where access to data is like:
    # image[:], image[] or image() will return full disk
    # image[2:56, 1020:1070]
    # image.area_extent(area_extent)
    #
    def __init__(self, config_reader):
        #
        # Read configuration file based on satellite name
        #
        sat = config_reader('satellite')
        projname = sat['projection'].lower()
        if not projname.startswith('geos'):
            raise mipp.ReaderError("currently we only support projections of type: 'GEOS'")

        #
        # Load format decoder based on level1 format
        #
        format = config_reader('level1')['format']
        try:
            args = imp.find_module(format)
        except ImportError:
            raise mipp.ReaderError("unknown level-1 format: '%s'"%format)
        try:
            m = imp.load_module(format, *args)
        finally:
            if args[0]:
                args[0].close()

        self._metadata_reader = m.read_metadata

        #
        # Attributing
        #
        self.__dict__.update(sat)

        self._config_reader = config_reader
        self.satname = self.satname + self.number
        self.satnumber = self.number
        delattr(self, 'number')

        # backwards compatible
        if not hasattr(self, 'proj4_params'):
            try:
                sublon = float(projname.split('(')[1].split(')')[0])
            except (IndexError, ValueError):
                raise mipp.ReaderError("Could not determine sub satellite point from projection name '%s'"%
                                       projname)
            self.proj4_params = "proj=geos lon_0=%.2f lat_0=0.00 a=6378169.00 b=6356583.80 h=35785831.00"%sublon            

    def load(self, time_stamp, channel, **kwarg):
        if channel not in self._config_reader.channel_names:
            raise mipp.ReaderError("unknown channel name '%s'"%channel)
        opt = self._config_reader('level1')
        val = {}
        val["channel"] = channel + '*'

        # Prologue
        
        val["segment"] = "PRO".ljust(9, '_')

        filename_pro = opt.get('filename_pro', opt['filename'])
        prologue = glob.glob(opt['dir'] + '/' + \
                             (time_stamp.strftime(filename_pro)%val))
        if not prologue:
            raise mipp.NoFiles("missing prologue file: '%s'"%(time_stamp.strftime(filename_pro)%val))
        prologue = prologue[0]

        # Regular channels
           
        val["segment"] = "0????????"
        image_files = glob.glob(opt['dir'] + '/' + \
                                time_stamp.strftime(opt['filename'])%val)
        if not image_files:
            raise mipp.NoFiles("no data files: '%s'"%(time_stamp.strftime(opt['filename'])%val))
        image_files.sort()

        logger.info("Read %s"%prologue)
        prologue = _xrit.read_prologue(prologue)

        # Epilogue

        val["segment"] = "EPI".ljust(9, '_')

        filename_epi = opt.get('filename_pro', opt['filename'])
        epilogue = glob.glob(opt['dir'] + '/' + \
                             (time_stamp.strftime(filename_epi)%val))

        if not epilogue:
            logger.info("No epilogue file to read.")
        else:
            epilogue = epilogue[0]
            logger.info("Read %s"%epilogue)
            epilogue = _xrit.read_epilogue(epilogue)
            return self.load_files(prologue, image_files,
                                   epilogue=epilogue, **kwarg)
        
        return self.load_files(prologue, image_files, **kwarg)

    def load_files(self, prologue, image_files, only_metadata=False, **kwargs):
        image_files.sort()
        if only_metadata:
            return self._read_metadata(prologue, image_files, **kwargs)
        else:
            return self._read(prologue, image_files, **kwargs)
        

    def _read_metadata(self, prologue, image_files, epilogue=None):
        if epilogue:
            mda = self._metadata_reader(prologue, image_files, epilogue=epilogue)
        else:
            mda = self._metadata_reader(prologue, image_files)
        if "%.2f"%mda.sublon != "%.2f"%self.sublon:
            if CHECK_CONFIG_SUBLON:
                raise mipp.ReaderError("Sub satellite point in config file (%.2f) don't match data (%.2f)"%
                                          (self.sublon, mda.sublon))
            else:
                logger.warning("Modifying sub satellite point from %.2f to %.2f"%
                               (self.sublon, mda.sublon))
                self.sublon = mda.sublon
                
        
        chn = self._config_reader.get_channel(mda.channel)
        if mda.image_size[0] != chn.size[0]:
            raise mipp.ReaderError("unknown image width for %s, %s: %d"%
                                   (self.satname, mda.channel, mda.image_size[0]))
                                
        mda.pixel_size = numpy.array([chn.resolution, chn.resolution], dtype=numpy.float64)
        for k, v in self.__dict__.items():
            if k[0] != '_' and type(v) != types.FunctionType:
                setattr(mda, k, v)
                
        img = _xrit.read_imagedata(image_files[0])
        
        return mda

    def _read(self, prologue, image_files, epilogue=None, **kwargs):
        if epilogue:
            mda = self._read_metadata(prologue, image_files, epilogue=epilogue)
        else:
            mda = self._read_metadata(prologue, image_files)
	len_img = (((mda.image_size[0] + mda.line_offset)*mda.image_size[1])*mda.data_type)//8
        logger.info("Data size: %dx%d pixels, %d bytes, %d bits per pixel",
                    mda.image_size[0], mda.image_size[1], len_img, mda.data_type)

        #
        # Return a proxy slicer
        #
        return ImageLoader(mda, image_files, **kwargs)
            
    #
    # Manipulate proj4's lon_0 parameter
    #
    _sublon_re = re.compile('(lon_0)=(\S+)')
    def _get_sublon(self):
        m = self._sublon_re.search(self.proj4_params)
        if m:
            return float(m.group(2))
        raise TypeError, "'SatelliteLoader' object (attribute proj4_params) has no 'sublon' attribute"
    def _set_sublon(self, slon):
        slon = "lon_0=%.2f"%float(slon)
        p = self.proj4_params
        m = self._sublon_re.search(p)
        if m:
            self.proj4_params = p.replace(m.group(0), slon)
        else:
            self.proj4_params += " %s"%slon
    sublon = property(_get_sublon, _set_sublon)

#-----------------------------------------------------------------------------
#
# Interface
#
#-----------------------------------------------------------------------------
def load_files(prologue, image_files, epilogue=None, **kwarg):
    if type(prologue) == type('string'):
        logger.info("Read %s"%prologue)
        prologue = _xrit.read_prologue(prologue)
    if epilogue and type(epilogue) == type('string'):
        logger.info("Read %s"%epilogue)
        epilogue = _xrit.read_epilogue(epilogue)
    satname = prologue.platform.lower()
    return SatelliteLoader(mipp.cfg.read_config(satname)).load_files(prologue, 
                                                                     image_files, 
                                                                     epilogue=epilogue, 
                                                                     **kwarg)

def load(satname, time_stamp, channel, **kwarg):
    return SatelliteLoader(mipp.cfg.read_config(satname)).load(time_stamp, channel, **kwarg)
 
def load_meteosat07(time_stamp, channel, **kwarg):
    return load('meteosat07', time_stamp, channel, **kwarg)
 
def load_meteosat09(time_stamp, channel, **kwarg):
    return load('meteosat09', time_stamp, channel, **kwarg)

def load_goes11(time_stamp, channel, **kwarg):
    return load('goes11', time_stamp, channel, **kwarg)
 
def load_goes12(time_stamp, channel, **kwarg):
    return load('goes12', time_stamp, channel, **kwarg)
 
def load_goes13(time_stamp, channel, **kwarg):
    return load('goes13', time_stamp, channel, **kwarg)
 
def load_mtsat1r(time_stamp, channel, **kwarg):
    return load('mtsat1r', time_stamp, channel, **kwarg)

def load_mtsat2(time_stamp, channel, **kwarg):
    return load('mtsat2', time_stamp, channel, **kwarg)

def load_electrol(time_stamp, channel, **kwarg):
    return load('electrol', time_stamp, channel, **kwarg)

#-----------------------------------------------------------------------------



#
# $id$
#
# Inspired by NWCLIB
#
import numpy
import types
import copy

import logging
logger = logging.getLogger('mipp')

import mipp
from mipp.xrit import _xrit, convert

__all__ = ['ImageLoader']

def _null_converter(blob):
    return blob

class ImageLoader(object):
    
    def __init__(self, mda, image_files, mask=False, calibrate=False):
        self.mda = mda
        self.image_files = image_files
        self.do_mask = mask        
        self.do_calibrate = calibrate
        # full disc and square
        self._allrows = slice(0, self.mda.image_size[0]) # !!!
        self._allcolumns = slice(0, self.mda.image_size[0])        

    def raw_slicing(self, item):
        """Raw slicing, no rotation of image.
        """
        # All data reading should end up here.

        # Don't mess with callers metadata.
        mda = copy.copy(self.mda)
        rows, columns = self._handle_item(item)

        ns_, ew_ = mda.first_pixel.split()

        if not hasattr(mda, "boundaries"):
            image = self._read(rows, columns, mda)

        else:
            #
            # Here we handle the case of partly defined channels.
            # (for example MSG's HRV channel)
            #
            image = None

            offset = 0
            offset_position = 0

            for region in (mda.boundaries - 1):
                offset += region[0] - offset_position
                offset_position = region[1] + 1

                rlines = slice(region[0], region[1] + 1)
                rcols = slice(region[2], region[3] + 1)

                # check is we are outside the region
                if (rows.start > rlines.stop or
                    rows.stop < rlines.start or
                    columns.start > rcols.stop or
                    columns.stop < rcols.start):
                    continue

                lines = slice(max(rows.start, rlines.start) - offset,
                              min(rows.stop, rlines.stop) - offset)
                cols =  slice(max(columns.start, rcols.start) - rcols.start,
                              min(columns.stop, rcols.stop) - rcols.start)
                rdata = self._read(lines, cols, mda)
                lines = slice(max(rows.start, rlines.start) - rows.start,
                              min(rows.stop, rlines.stop) - rows.start)
                cols =  slice(max(columns.start, rcols.start) - columns.start,
                              min(columns.stop, rcols.stop) - columns.start)
                if image is None:
                    image = (numpy.zeros((rows.stop - rows.start,
                                          columns.stop - columns.start),
                                         dtype=rdata.dtype)
                             + mda.no_data_value)
                    if self.do_mask:
                        image = numpy.ma.masked_all_like(image)

                if ns_ == "south":
                    lines = slice(image.shape[0] - lines.stop,
                                  image.shape[0] - lines.start)
                if ew_ == "east":
                    cols = slice(image.shape[1] - cols.stop,
                                 image.shape[1] - cols.start)
                if self.do_mask:
                    image.mask[lines, cols] = rdata.mask
                image[lines, cols] = rdata

        if not hasattr(image, 'shape'):
            logger.warning("Produced no image")
            return None, None

        #
        # Update meta-data
        #
        mda.area_extent = numpy.array(self._slice2extent(rows, columns, rotated=True), dtype=numpy.float64)

        if (rows != self._allrows) or (columns != self._allcolumns):
            mda.region_name = 'sliced'

        mda.data_type = 8*image.itemsize
        mda.image_size = numpy.array([image.shape[1], image.shape[0]])

        return mipp.mda.mslice(mda), image
    
    def __getitem__(self, item):
        """Deafult slicing, handles rotated images.
        """
        rows, columns = self._handle_item(item)
        ns_, ew_ = self.mda.first_pixel.split()
        if ns_ == 'south':
            rows = slice(self.mda.image_size[1] - rows.stop,
                         self.mda.image_size[1] - rows.start)
        if ew_ == 'east':
            columns = slice(self.mda.image_size[0] - columns.stop,
                            self.mda.image_size[0] - columns.start)
        return self.raw_slicing((rows, columns))

    def __call__(self, area_extent=None):
        """Slice according to (ll_x, ll_y, ur_x, ur_y) or read full disc.
        """
        if area_extent == None:
            # full disc
            return self[:]
            
        # slice
        area_extent = tuple(area_extent)
        if len(area_extent) != 4:
            raise TypeError, "optional argument must be an area_extent"

        ns_, ew_ = self.mda.first_pixel.split()

        if ns_ == "south":
            loff = self.mda.image_size[0] - self.mda.loff - 1
        else:
            loff = self.mda.loff - 1

        if ew_ == "east":
            coff = self.mda.image_size[1] - self.mda.coff - 1
        else:
            coff = self.mda.coff - 1

            
        row_size = self.mda.pixel_size[0]
        col_size = self.mda.pixel_size[1]

        logger.debug('area_extent: %.2f, %.2f, %.2f, %.2f'%tuple(area_extent))
        logger.debug('area_extent: resolution %.2f, %.2f'%(row_size, col_size))
        logger.debug('area_extent: loff, coff %d, %d'%(loff, coff))
        logger.debug('area_extent: expected size %d, %d'%\
                         (int(numpy.round((area_extent[2] - area_extent[0])/col_size)),\
                         int(numpy.round((area_extent[3] - area_extent[1])/row_size))))
        
        col_start = int(numpy.round(area_extent[0] / col_size + coff + 0.5))
        row_stop = int(numpy.round(area_extent[1] / -row_size + loff - 0.5))
        col_stop = int(numpy.round(area_extent[2] / col_size + coff - 0.5))
        row_start = int(numpy.round(area_extent[3] / -row_size + loff + 0.5))

        row_stop += 1
        col_stop += 1

        logger.debug('area_extent: computed size %d, %d'%(col_stop - col_start, row_stop - row_start))

        return self[row_start:row_stop, col_start:col_stop]

    def _handle_item(self, item):
        """Transform item into slice(s).
        """
        if isinstance(item, slice):
            # specify rows and all columns
            rows, columns = item, self._allcolumns
        elif isinstance(item, int):
            # specify one row and all columns
            rows, columns = slice(item, item + 1), self._allcolumns
        elif isinstance(item, tuple):
            if len(item) == 2:
                # both row and column are specified 
                rows, columns = item
                if isinstance(rows, int):
                    rows = slice(item[0], item[0] + 1)
                if isinstance(columns, int):
                    columns = slice(item[1], item[1] + 1)
            else:
                raise IndexError, "can only handle two indexes, not %d"%len(item)
        elif item == None:
            # full disc
            rows, columns = self._allrows, self._allcolumns            
        else:
            raise IndexError, "don't understand the indexes"

        # take care of [:]
        if rows.start == None:
            rows = self._allrows
        if columns.start == None:
            columns = self._allcolumns
            
        if (rows.step != 1 and rows.step != None) or \
               (columns.step != 1 and columns.step != None):
            raise IndexError, "Currently we don't support steps different from one"

        return rows, columns

    def _slice2extent(self, rows, columns, rotated=True):
        """ Calculate area extent.
        If rotated=True then rows and columns are reflecting the actual rows and columns.
        """
        ns_, ew_ = self.mda.first_pixel.split()

        loff = self.mda.loff
        coff = self.mda.coff
        if ns_ == "south":
            loff = self.mda.image_size[0] - loff - 1
            if rotated:
                rows = slice(self.mda.image_size[1] - rows.stop,
                             self.mda.image_size[1] - rows.start)
        else:
            loff -= 1
        if ew_ == "east":
            coff = self.mda.image_size[1] - coff - 1
            if rotated:
                columns = slice(self.mda.image_size[0] - columns.stop,
                                self.mda.image_size[0] - columns.start)
        else:
            coff -= 1

        logger.debug('slice2extent: size %d, %d'% \
                         (columns.stop - columns.start, rows.stop - rows.start))
        rows = slice(rows.start, rows.stop - 1)
        columns = slice(columns.start, columns.stop - 1)

        row_size = self.mda.pixel_size[0]
        col_size = self.mda.pixel_size[1]
      
        ll_x = (columns.start - coff - 0.5)*col_size
        ll_y = -(rows.stop - loff + 0.5)*row_size
        ur_x = (columns.stop - coff + 0.5)*col_size
        ur_y = -(rows.start - loff - 0.5)*row_size

        logger.debug('slice2extent: computed extent %.2f, %.2f, %.2f, %.2f'% \
                         (ll_x, ll_y, ur_x, ur_y))
        logger.debug('slice2extent: computed size %d, %d'% \
                         (int(numpy.round((ur_x - ll_x)/col_size)), \
                              int(numpy.round((ur_y - ll_y)/row_size))))

        return [ll_x, ll_y, ur_x, ur_y]

    def _read(self, rows, columns, mda):
        shape = (rows.stop - rows.start, columns.stop - columns.start)
        if (columns.start < 0 or
            columns.stop > mda.image_size[0] or
            rows.start < 0 or
            rows.stop > mda.image_size[1]):
            raise IndexError, "index out of range"

        image_files = self.image_files
        
        #
        # Order segments
        #
        segments = {}
        for f in image_files:
            s = _xrit.read_imagedata(f)
            segments[s.segment.seg_no] = f
        start_seg_no = s.segment.planned_start_seg_no
        end_seg_no = s.segment.planned_end_seg_no
        ncols =  s.structure.nc
        segment_nlines = s.structure.nl

        #
        # Data type
        #
        converter = _null_converter
        if mda.data_type == 8:        
            data_type = numpy.uint8
            data_type_len = 8
        elif mda.data_type == 10:
            converter = convert.dec10216
            data_type = numpy.uint16
            data_type_len = 16
        elif mda.data_type == 16:
            data_type = numpy.uint16
            data_type_len = 16
        else:
            raise mipp.ReaderError, "unknown data type: %d bit per pixel"\
                %mda.data_type

        #
        # Calculate initial and final line and column.
        # The interface 'load(..., center, size)' will produce
        # correct values relative to the image orientation. 
        # line_init, line_end : 1-based
        #
        line_init = rows.start + 1
        line_end = line_init + rows.stop - rows.start - 1
        col_count = shape[1]
        col_offset = (columns.start)*data_type_len//8

        #
        # Calculate initial and final segments
        # depending on the image orientation.
        # seg_init, seg_end : 1-based.
        #
        seg_init = ((line_init-1)//segment_nlines) + 1
        seg_end = ((line_end-1)//segment_nlines) + 1

        #
        # Calculate initial line in image, line increment
        # offset for columns and factor for columns,
        # and factor for columns, depending on the image
        # orientation
        #
        if mda.first_pixel == 'north west':
            first_line = 0
            increment_line = 1
            factor_col = 1
        elif mda.first_pixel == 'north east':
            first_line = 0
            increment_line = 1
            factor_col = -1
        elif mda.first_pixel == 'south west':
            first_line = shape[0] - 1
            increment_line = -1
            factor_col = 1
        elif mda.first_pixel == 'south east':
            first_line = shape[0] - 1
            increment_line = -1
            factor_col = -1
        else:
            raise mipp.ReaderError, "unknown geographical orientation of " + \
                "first pixel: '%s'"%mda.first_pixel

        #
        # Generate final image with no data
        #
        image = numpy.zeros(shape, dtype=data_type) + mda.no_data_value
    
        #
        # Begin the segment processing.
        #
        seg_no = seg_init
        line_in_image = first_line
        while seg_no <= seg_end:
            line_in_segment = 1
      
            #
            # Calculate initial line in actual segment.
            #
            if seg_no == seg_init:
                init_line_in_segment = (line_init
                                        - (segment_nlines*(seg_init - 1)))
            else:
                init_line_in_segment = 1

            #
            # Calculate final line in actual segment.
            #
            if seg_no == seg_end:
                end_line_in_segment = line_end - (segment_nlines*(seg_end - 1))
            else:
                end_line_in_segment = segment_nlines

            #
            # Open segment file.
            #
            seg_file = segments.get(seg_no, None)
            if not seg_file:
                #
                # No data for this segment.
                #
                logger.warning("Segment number %d not found"%seg_no)

                # all image lines are already set to no-data count.
                line_in_segment = init_line_in_segment
                while line_in_segment <= end_line_in_segment:
                    line_in_segment += 1
                    line_in_image += increment_line
            else:
                #
                # Data for this segment.
                #
                logger.info("Read %s"%seg_file)
                seg = _xrit.read_imagedata(seg_file)
            
                #
                # Skip lines not processed.
                #
                while line_in_segment < init_line_in_segment:
                    line = seg.readline()
                    line_in_segment += 1

                #
                # Reading and processing segment lines.
                #
                while line_in_segment <= end_line_in_segment:
                    line = seg.readline()[mda.line_offset:]
                    line = converter(line)

                    line = (numpy.frombuffer(line,
                                             dtype=data_type,
                                             count=col_count,
                                             offset=col_offset)[::factor_col])
                
                    #
                    # Insert image data.
                    #
                    image[line_in_image] = line
                
                    line_in_segment += 1
                    line_in_image += increment_line
            
                seg.close()

            seg_no += 1

        #
        # Compute mask before calibration
        #

        mask = (image == mda.no_data_value)

        #
        # Calibrate ?
        #
        mda.is_calibrated = False
        if self.do_calibrate:
            # do this before masking.
            calibrate = self.do_calibrate
            if isinstance(calibrate, bool):
                # allow boolean True/False for 1/0
                calibrate = int(calibrate)
            image, mda.calibration_unit = mda.calibrate(image, calibrate=calibrate)
            mda.is_calibrated = True

        #
        # With or without mask ?
        #
        if self.do_mask and not isinstance(image, numpy.ma.core.MaskedArray):
            image = numpy.ma.array(image, mask=mask, copy=False)
        elif ((not self.do_mask) and 
                isinstance(image, numpy.ma.core.MaskedArray)):
            image = image.filled(mda.no_data_value)
            
        return image




import os
import sys
from datetime import datetime
import numpy
import unittest

import buildpath_to_syspath
import mipp
from mipp import xrit
from mipp.mda import _nice2cmp

datadir = (os.path.dirname(__file__) or '.') + '/data'
save_mda = False
debug = os.environ.has_key('DEBUG')


try:
    # give the possibility to test other config files
    os.environ['PPP_CONFIG_DIR'] = os.environ['LOCAL_PPP_CONFIG_DIR']
except KeyError:
    os.environ['PPP_CONFIG_DIR'] = datadir
if not os.path.isdir(os.environ['PPP_CONFIG_DIR']):
    raise mipp.ConfigReaderError, "No config dir: '%s'"%os.environ['PPP_CONFIG_DIR']

goes_files = [datadir + '/L-000-MSG2__-GOES11______-10_7_135W-PRO______-201002010600-__',
              datadir + '/L-000-MSG2__-GOES11______-10_7_135W-000003___-201002010600-__',
              datadir + '/L-000-MSG2__-GOES11______-10_7_135W-000004___-201002010600-__']
goes_sum = 11629531.0

mtsat_files = [datadir + '/L-000-MSG2__-MTSAT1R_____-10_8_140E-PRO______-200912210900-__',
               datadir + '/L-000-MSG2__-MTSAT1R_____-10_8_140E-000003___-200912210900-__',
               datadir + '/L-000-MSG2__-MTSAT1R_____-10_8_140E-000004___-200912210900-__']
mtsat_sum = 11148074.0

met7_files = [datadir + '/L-000-MTP___-MET7________-00_7_057E-PRO______-200912211200-__',
              datadir + '/L-000-MTP___-MET7________-00_7_057E-000005___-200912211200-__',
              datadir + '/L-000-MTP___-MET7________-00_7_057E-000006___-200912211200-__']
met7_sum = 11662791

msg_files = [datadir + '/H-000-MSG2__-MSG2________-_________-PRO______-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-IR_108___-000004___-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-IR_108___-000005___-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-_________-EPI______-201010111400-__']
msg_sum = 75116847.263172984

hrv_files = [datadir + '/H-000-MSG2__-MSG2________-_________-PRO______-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-HRV______-000012___-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-HRV______-000013___-201010111400-__',
             datadir + '/H-000-MSG2__-MSG2________-_________-EPI______-201010111400-__']
hrv_sum = 11328340.753558

hrv2_files = [datadir + '/H-000-MSG2__-MSG2________-_________-PRO______-201011091200-__',
              datadir + '/H-000-MSG2__-MSG2________-HRV______-000018___-201011091200-__',
              datadir + '/H-000-MSG2__-MSG2________-_________-EPI______-201011091200-__']
hrv2_sum = 44049589.065626

def make_image(mda, img, outdir='.'):
    if not debug:
        return
    import Image as pil
    fname = outdir + '/' + mda.product_name + '.png'
    img = ((img - img.min()) * 255.0 /
           (img.max() - img.min()))
    if type(img) == numpy.ma.MaskedArray:
        img = img.filled(mda.no_data_value)
    img = pil.fromarray(numpy.array(img, numpy.uint8))
    img.save(fname)

def compare_mda(m1, m2):
    def compare_arrays(e1, e2):
        for x1, x2 in zip(e1, e2):
            if "%.3f"%x1 != "%.3f"%x2:
                return False
        return True
            
    k1 = sorted(m1.__dict__.keys())
    k2 = sorted(m2.__dict__.keys())
    if not k1 == k2:
        return False
    for k in k1:
        if k in ('area_extent', 'pixel_size'):
            if not compare_arrays(getattr(m1, k), getattr(m2, k)):
                return False
        elif not _nice2cmp(getattr(m1, k)) == _nice2cmp(getattr(m2, k)):
            return False
            
    return True

class Test(unittest.TestCase):

    def test_goes(self):
        loader = xrit.sat.load_files(goes_files[0], goes_files[1:], calibrate=True)
        mda, img = loader[1308:1508,1308:1508]
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        mdac.data_type = 8*img.itemsize
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='GOES metadata differ')
        self.assertTrue(img.shape == (200, 200), msg='GOES image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, goes_sum, 3,
                                   msg='GOES image reading/slicing failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, goes_sum))

    def test_mtsat(self):
        loader = xrit.sat.load_files(mtsat_files[0], mtsat_files[1:], calibrate=True)
        mda, img = loader[1276:1476,1276:1476]
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        mdac.data_type = 8*img.itemsize
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='MTSAT metadata differ')
        self.assertTrue(img.shape == (200, 200), msg='MTSAT image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, mtsat_sum, 3,
                                   msg='MTSAT image reading/slicing failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, mtsat_sum))

    def test_met7(self):
        loader = xrit.sat.load_files(met7_files[0], met7_files[1:], calibrate=False)
        mda, img = loader.raw_slicing((slice(2300,2900), slice(2000,3000)))
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='MET7 metadata differ')
        self.assertTrue(img.shape == (600, 1000), msg='MET7 image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, met7_sum, 3,
                                   msg='MET7 image reading/slicing failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, met7_sum))

    def test_msg(self):
        loader = xrit.sat.load_files(msg_files[0], msg_files[1:-1], epilogue=msg_files[-1], 
                                     calibrate=True)
        mda, img = loader[1656:1956,1756:2656]
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        mdac.data_type = 8*img.itemsize
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='MSG metadata differ')
        self.assertTrue(img.shape == (300, 900), msg='MSG image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, msg_sum, 3,
                                   msg='MSG image reading/slicing reflectances failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, msg_sum))

        mda, img = loader(mda.area_extent)
        if save_mda:
            mda.save(mda.product_name + '.mda')
        cross_sum = img.sum()
        self.assertTrue(compare_mda(mda, mdac), msg='MSG metadata differ, when using area_extent')
        self.failUnlessAlmostEqual(cross_sum, msg_sum, 3,
                                   msg='MSG image reading/slicing failed, when using area_extent, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, msg_sum))

    def test_msg2(self):
        loader = xrit.sat.load_files(msg_files[0], msg_files[1:-1], epilogue=msg_files[-1], 
                                     calibrate=2)
        mda, img = loader[1656:1956,1756:2656]
        cross_sum = img.sum()
        expected = 22148991.0194
        self.failUnlessAlmostEqual(cross_sum, expected, 3,
                                   msg='MSG image reading/slicing radiances failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, expected))
        
        
    def test_hrv(self):
        loader = xrit.sat.load_files(hrv_files[0], hrv_files[1:-1], epilogue=hrv_files[-1], calibrate=True)
        mda, img = loader[5168:5768,5068:6068]
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        mdac.data_type = 8*img.itemsize
        cross_sum = img.sum()
        make_image(mda, img)
        self.assertTrue(compare_mda(mda, mdac), msg='MSG-HRV metadata differ')
        self.assertTrue(img.shape == (600, 1000), msg='MSG-HRV image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, hrv_sum, 3,
                                   msg='MSG-HRV image reading/slicing failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, hrv_sum))
    
    def test_hrv2(self):
        loader = xrit.sat.load_files(hrv2_files[0], hrv2_files[1:-1], epilogue=hrv2_files[-1], calibrate=True)
        mda, img = loader[2786:3236,748:9746]
        if save_mda:
            mda.save(mda.product_name + '.mda')
        mdac = xrit.Metadata().read(datadir + '/' + mda.product_name + '.mda')
        mdac.data_type = 8*img.itemsize
        cross_sum = img.sum()        
        make_image(mda, img)
        
        self.assertTrue(compare_mda(mda, mdac), msg='MSG-HRV metadata differ')
        self.assertTrue(img.shape == (450, 8998), msg='MSG-HRV image reading/slicing failed, wrong shape')
        self.failUnlessAlmostEqual(cross_sum, hrv2_sum, 3,
                                   msg='MSG-HRV image reading/slicing failed, wrong cross_sum (%.3f != %.3f)'%(
                cross_sum, hrv2_sum))

if __name__ == '__main__':
    save_mda = False
    unittest.main()



#
# $Id$ 
#
import os
import tarfile
import glob
import fnmatch
import imp

import logging
logger = logging.getLogger('mipp')

import mipp
import mipp.cfg

__all__ = ['load']

def _find_in_tarfile(tfile, fname):
    tar = tarfile.open(tfile)
    try:
        for name in tar.getnames():
            if name.endswith(fname):
                return name
    finally:
        tar.close()
    raise mipp.NoFiles("found no archive file '%s'"%fname)

class SatelliteLoader(object):
    def __init__(self, config_reader):
        #
        # Read configuration file based on satellite name
        #
        sat = config_reader('satellite')

        #
        # Load format decoder based on level1 format
        #
        lv1_format = config_reader('level1')['format']
        logger.info("Loading %s"%lv1_format)
        try:
            _args = imp.find_module(lv1_format)
        except ImportError:
            raise mipp.ReaderError("unknown level-1 format: '%s'"%lv1_format)
        try:
            mdl = imp.load_module(lv1_format, *_args)
        finally:
            if _args[0]:
                _args[0].close()

        self._metadata_reader = mdl.read_metadata
        self._image_reader = mdl.read_image
        self._tar_file = None

        #
        # Attributing
        #
        self.__dict__.update(sat)

        self._config_reader = config_reader
        self.satname = self.satname + self.number
        self.satnumber = self.number
        del self.number

    def load(self, time_stamp, channel, **kwargs):
        return self.load_file(self._find_tarfile(time_stamp),
                              channel, **kwargs)

    def load_file(self, filename, channel,
                  only_metadata=False, mask=True, calibrate=1):
        if channel not in self._config_reader.channel_names:
            raise mipp.ReaderError("unknown channel name '%s'"%channel)
        self._tar_file = filename
        mda = self._load_metadata(channel)
        if only_metadata:
            return mda
        mda, img = self._load_image(mda, mask=mask, calibrate=calibrate)        
        return mipp.mda.mslice(mda), img

        
    def _load_metadata(self, channel):
        del channel
        opt = self._config_reader('level1')
        mda_file = opt['filename_metadata']
        tar = tarfile.open(self._tar_file)
        names = []
        try:
            for name in tar.getnames():
                if fnmatch.fnmatch(os.path.basename(name), mda_file):
                    names.append(name)
            if len(names) == 0:
                raise mipp.NoFiles("found no metadata file: '%s'"%mda_file)
            elif len(names) > 1:
                raise mipp.NoFiles("found multiple metadata files: '%s'" %
                                   str(names))
            logger.info("Extracting '%s'"%names[0])
            xmldata = tar.extractfile(names[0]).read()
        finally:
            tar.close()
        return self._metadata_reader(xmldata)

    def _load_image(self, mda, mask=True, calibrate=1):
        import tempfile
        import shutil
        image_file = _find_in_tarfile(self._tar_file, mda.image_filename)
        tar = tarfile.open(self._tar_file)
        tmpdir = tempfile.mkdtemp()
        logger.info("Extracting '%s' into '%s'"%(image_file, tmpdir))
        try:
            tar.extract(image_file, tmpdir)
            image_file = tmpdir + '/' + mda.image_filename
            mda, image = self._image_reader(mda, image_file,
                                            mask=mask, calibrate=calibrate)
        finally:
            tar.close()
            shutil.rmtree(tmpdir)
        return mda, image

    def _find_tarfile(self, time_stamp):
        opt = self._config_reader('level1')
        if not os.path.isdir(opt['dir']):
            raise IOError, "No such directory: %s" % opt['dir']
        tar_file = glob.glob(opt['dir'] + '/' +
                             time_stamp.strftime(opt['filename_archive']))
        if not tar_file:
            raise mipp.NoFiles("found no archive file: '%s'"%
                               (time_stamp.strftime(opt['filename_archive'])))
        elif len(tar_file) > 1:
            raise mipp.NoFiles("found multiple archive files: '%s'" % 
                               str(tar_file))
        return tar_file[0]

#-----------------------------------------------------------------------------
#
# Interface
#
#-----------------------------------------------------------------------------
def load(satname, time_stamp, channel, **kwarg):
    return SatelliteLoader(
        mipp.cfg.read_config(satname)).load(time_stamp, channel, **kwarg)

def load_file(filename, channel, **kwarg):
    # Satellite name should be read from metadata (and not filename) !!!
    satname = os.path.basename(filename).split('_')[0].lower()
    return SatelliteLoader(
        mipp.cfg.read_config(satname)).load_file(filename, channel, **kwarg)

#-----------------------------------------------------------------------------

if __name__ == '__main__':
    import sys
    import getopt
    import loggers.simple
    logger = loggers.simple.get('mipp')

    only_mda = False
    opts, args = getopt.getopt(sys.argv[1:], "m")
    for k, v in opts:
        if k == '-m':
            only_mda = True
    try:
        _file = args[0]
    except IndexError:
        print >> sys.stderr, "usage: sat.py [-m] <tar-file>"
        sys.exit(1)
    _mda = load_file(_file, 'sarx', only_metadata=True)
    if only_mda:
        pass
    elif _mda.calibrated != 'CALIBRATED':
        logger.warning("Data is not calibrated")
        _mda, _image = load_file(_file, 'sarx', calibrate=0)
    else:
        _mda, _image = load_file(_file, 'sarx')
    print '\n', _mda



#
# $Id$
#
import mipp.mda

class Metadata(mipp.mda.Metadata):
    token = ':'
    ignore_attributes = ('data',
                         'calibrate',
                         'tiepoints')

if __name__ == '__main__':
    import sys
    print Metadata().read(sys.argv[1])



import sys
import os
from distutils.util import get_platform

proj_path = os.path.abspath(os.path.join(os.path.dirname(__file__),'..'))
build_base = 'build'
plat = get_platform()

python_version = '.'.join([str(i) for i in sys.version_info[:2]])
build_purelib = os.path.join(build_base, 'lib' + '-' + python_version) # is it at all used ?
build_platlib = os.path.join(build_base, 'lib.' + plat + '-' + python_version)

build_path = os.path.join(proj_path, build_platlib)
if not os.path.isdir(build_path):
    assert False, "No such build path '%s'"%build_path
sys.path.insert(0, build_path)

if __name__ == '__main__':
    print build_platlib



#
# WORK IN PROGRESS for CosmoSkyMed
#
import os
import copy
import numpy as np
from datetime import datetime, timedelta
from lxml import etree

import logging
logger = logging.getLogger('mipp')

import mipp
from mipp import geotiff
from mipp.xsar import Metadata

__all__ = ['read_metadata', 'read_image']

class _Calibrator(object):
    def __init__(self, mda):
        self.error = "Unknown calibration"

    def __call__(self, image, calibrate=1):
        if calibrate == 0:
            return (image,
                    'counts')
        raise mipp.CalibrationError, self.error

def read_metadata(xmlbuffer):
    mda = Metadata()    

    # Speciel decoders
    def dec_timeformat(strn):
        strn = strn.split('.')        
        return (datetime.strptime(strn[0], "%Y-%m-%d %H:%M:%S") + 
                timedelta(seconds=float('.' + strn[1])))
    def dec_orbit_number(strn):
        return int(strn[:5])

    attributes = (
        ('_ROOT_/Attribute', {
                'Satellite ID': ('satellite_name', str),
                'Product Filename': ('image_filename', str),
                'Product Type': ('product_type', str),
                'Acquisition Station ID': ('facility_id', str),
                'Scene Sensing Start UTC': ('time_start', dec_timeformat),
                'Scene Sensing Stop UTC': ('time_stop', dec_timeformat),
                'Orbit Number': ('orbit_number', dec_orbit_number),
                'Sample Format': ('product_format', str),
                'Image Scale': ('image_scale', str),
                'Image Layers': ('layers', int),
                'Bits per Sample': ('bits_per_sample', int),
                'Samples per Pixel': ('samples_per_pixel', int),
                }),
        
        ('MBI/Attribute', {
                'Column Spacing': ('sample_spacing', float),
                'Line Spacing': ('line_spacing', float)
                }),

        ('S01/Attribute', {            
                'Polarisation': ('polarisation', str),
                }),
        )

    tree = etree.fromstring(xmlbuffer)

    #
    # Get Atrributes
    #
    for path, attr in attributes:
        names = attr.keys()
        path = tree.xpath(path)
        for i in path:
            name = i.attrib['Name']
            if name in names:
                names.remove(name)
                val = i.text
                setattr(mda, attr[name][0], attr[name][1](val))

    

    satid = 'CSK'
    if not mda.satellite_name.upper().startswith(satid):
        raise mipp.ReaderError(
            "This does not look like a CosmoSkymed product, " + 
            "satellite ID does now start with '%s'"%satid)

    mda.image_filename = os.path.splitext(mda.image_filename)[0] + '.MBI.tif'

    mda.no_data_value = 0
    mda.calibrated = 'NOTCALIBRATED'

    return mda

def read_image(mda, filename=None, mask=True, calibrate=1):
    mda = copy.copy(mda)
    mda.calibrate = _Calibrator(mda)
    mda.calibration_unit = 'counts'
    mda.is_calibrated = False
    del mda.calibrated
    mda.product_name = (mda.time_start.strftime("%Y%m%d_%H%M%S") + '_' + 
                        mda.satellite_name + '_' +  
                        mda.product_type + '_' +
                        mda.polarisation)
    logger.info('Product name: %s'% mda.product_name)
    
    if not filename:
        filename = mda.image_filename

    params, data = geotiff.read_geotiff(filename)
    area_def = geotiff.tiff2areadef(params['projection'],
                                    params['geotransform'],
                                    data.shape)

    mda.proj4_params = area_def.proj4_string.replace('+', '')
    mda.area_extent = area_def.area_extent
    mda.tiff_params = params

    if calibrate:
        data, mda.calibration_unit = mda.calibrate(data, calibrate)
        mda.is_calibrated = True
        logger.info('calibrated: %s %s [%.2f -> %.2f -> %.2f] %s'%
                    (str(data.shape), data.dtype, data.min(),
                     data.mean(), data.max(), mda.calibration_unit))

    if mask:
        mask = (data == mda.no_data_value)
        data = np.ma.array(data, mask=mask, copy=False)

    return mda, data

if __name__ == '__main__':
    import sys
    with open(sys.argv[1]) as fp:
        _mda = read_metadata(fp.read())
    _mda.image_filename = os.path.dirname(sys.argv[1]) + '/' + \
        _mda.image_filename        
    print _mda
    _mda, _data = read_image(_mda, calibrate=False)
    print _data.min(), _data.mean(), _data.max()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010, 2011, 2012, 2013

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>
#   Lars Ø. Rasmusen <ras@dmi.dk>
#   Esben S. Nielsen <esn@dmi.dk>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module will read MSG level1.5 files, format documented in: 
'MSG Level 1.5 Image Data Format Description', EUM/MSG/ICD/105, v5A, 22 August 2007
"""
#raise NotImplementedError
import logging
logger = logging.getLogger('mipp')

import sys
import numpy as np
from StringIO import StringIO

from mipp import CalibrationError
from mipp.xrit import _xrit
from mipp.xrit import Metadata
from mipp.xrit import bin_reader as rbin

__all__ = ['read_metadata']

eval_np = eval
log = np.log
no_data_value = 0

if sys.version_info[0] >= 2 and sys.version_info[1] >= 5:
    try:
        #Use numexpr if available
        import numexpr
        eval_np = numexpr.evaluate
        logger.info('Using numexpr for fast numpy evaluation')
    except ImportError:
        logger.warning('Module numexpr not found. Performance will be slower.')
else:
    logger.warning('Older version of python. Module numexpr not used. '
                   'Performance will be slower.')
        

#Reflectance factor for visible bands
HRV_F    = 25.15
VIS006_F = 20.76
VIS008_F = 23.30
IR_016_F = 19.73

## Calibration coefficients from
##'A Planned Change to the MSG Level 1.5 Image Product Radiance Definition'
## ,
## "Conversion from radiances to reflectances for SEVIRI warm channels"
## EUM/MET/TEN/12/0332
## , and
## "The Conversion from Effective Radiances to Equivalent Brightness
## Temperatures"
## EUM/MET/TEN/11/0569

CALIB = {}


# Meteosat 8

CALIB[321] = {'HRV': {'F': 78.7599 / np.pi},
              'VIS006': {'F': 65.2296 / np.pi},
              'VIS008': {'F': 73.0127 / np.pi},
              'IR_016': {'F': 62.3715 / np.pi},
              'IR_039': {'VC': 2567.33,
                         'ALPHA': 0.9956,
                         'BETA': 3.41},
              'WV_062': {'VC': 1598.103,
                         'ALPHA': 0.9962,
                         'BETA': 2.218},
              'WV_073': {'VC': 1362.081,
                         'ALPHA': 0.9991,
                         'BETA': 0.478},
              'IR_087': {'VC': 1149.069,
                         'ALPHA': 0.9996,
                         'BETA': 0.179},
              'IR_097': {'VC': 1034.343,
                         'ALPHA': 0.9999,
                         'BETA': 0.06},
              'IR_108': {'VC': 930.647,
                         'ALPHA': 0.9983,
                         'BETA': 0.625},
              'IR_120': {'VC': 839.66,
                         'ALPHA': 0.9988,
                         'BETA': 0.397},
              'IR_134': {'VC': 752.387,
                         'ALPHA': 0.9981,
                         'BETA': 0.578}}

# Meteosat 9

CALIB[322] = {'HRV': {'F': 79.0113 / np.pi},
              'VIS006': {'F': 65.2065 / np.pi},
              'VIS008': {'F': 73.1869 / np.pi},
              'IR_016': {'F': 61.9923 / np.pi},
              'IR_039': {'VC': 2568.832,
                         'ALPHA': 0.9954,
                         'BETA': 3.438},
              'WV_062': {'VC': 1600.548,
                         'ALPHA': 0.9963,
                         'BETA': 2.185},
              'WV_073': {'VC': 1360.330,
                         'ALPHA': 0.9991,
                         'BETA': 0.47},
              'IR_087': {'VC': 1148.620,
                         'ALPHA': 0.9996,
                         'BETA': 0.179},
              'IR_097': {'VC': 1035.289,
                         'ALPHA': 0.9999,
                         'BETA': 0.056},
              'IR_108': {'VC': 931.7,
                         'ALPHA': 0.9983,
                         'BETA': 0.64},
              'IR_120': {'VC': 836.445,
                         'ALPHA': 0.9988,
                         'BETA': 0.408},
              'IR_134': {'VC': 751.792,
                         'ALPHA': 0.9981,
                         'BETA': 0.561}}

# Meteosat 10

CALIB[323] = {'HRV': {'F': 78.9416 / np.pi},
              'VIS006': {'F': 65.5148 / np.pi},
              'VIS008': {'F': 73.1807 / np.pi},
              'IR_016': {'F': 62.0208 / np.pi},
              'IR_039': {'VC': 2547.771,
                         'ALPHA': 0.9915,
                         'BETA': 2.9002},
              'WV_062': {'VC': 1595.621,
                         'ALPHA': 0.9960,
                         'BETA': 2.0337},
              'WV_073': {'VC': 1360.337,
                         'ALPHA': 0.9991,
                         'BETA': 0.4340},
              'IR_087': {'VC': 1148.130,
                         'ALPHA': 0.9996,
                         'BETA': 0.1714},
              'IR_097': {'VC': 1034.715,
                         'ALPHA': 0.9999,
                         'BETA': 0.0527},
              'IR_108': {'VC': 929.842,
                         'ALPHA': 0.9983,
                         'BETA': 0.6084},
              'IR_120': {'VC': 838.659,
                         'ALPHA': 0.9988,
                         'BETA': 0.3882},
              'IR_134': {'VC': 750.653,
                         'ALPHA': 0.9982,
                         'BETA': 0.5390}}

# Meteosat 11

CALIB[324] = {'HRV': {'F': 79.0035/ np.pi},
              'VIS006': {'F': 65.2656 / np.pi},
              'VIS008': {'F': 73.1692 / np.pi},
              'IR_016': {'F': 61.9416 / np.pi},
              'IR_039': {'VC': 2555.280,
                         'ALPHA': 0.9916,
                         'BETA': 2.9438},
              'WV_062': {'VC': 1596.080,
                         'ALPHA': 0.9959,
                         'BETA': 2.0780},
              'WV_073': {'VC': 1361.748,
                         'ALPHA': 0.9990,
                         'BETA': 0.4929},
              'IR_087': {'VC': 1147.433,
                         'ALPHA': 0.9996,
                         'BETA': 0.1731},
              'IR_097': {'VC': 1034.851,
                         'ALPHA': 0.9998,
                         'BETA': 0.0597},
              'IR_108': {'VC': 931.122,
                         'ALPHA': 0.9983,
                         'BETA': 0.6256},
              'IR_120': {'VC': 839.113,
                         'ALPHA': 0.9988,
                         'BETA': 0.4002},
              'IR_134': {'VC': 748.585,
                         'ALPHA': 0.9981,
                         'BETA': 0.5635}}

#Polynomial coefficients for spectral-effective BT fits
BTFIT_A_IR_039 =  0.0
BTFIT_A_WV_062 =  0.00001805700
BTFIT_A_WV_073 =  0.00000231818
BTFIT_A_IR_087 = -0.00002332000
BTFIT_A_IR_097 = -0.00002055330 
BTFIT_A_IR_108 = -0.00007392770
BTFIT_A_IR_120 = -0.00007009840
BTFIT_A_IR_134 = -0.00007293450

BTFIT_B_IR_039 =  1.011751900
BTFIT_B_WV_062 =  1.000255533 
BTFIT_B_WV_073 =  1.000668281
BTFIT_B_IR_087 =  1.011803400
BTFIT_B_IR_097 =  1.009370670  
BTFIT_B_IR_108 =  1.032889800 
BTFIT_B_IR_120 =  1.031314600
BTFIT_B_IR_134 =  1.030424800

BTFIT_C_IR_039 =  -3.550400
BTFIT_C_WV_062 =  -1.790930
BTFIT_C_WV_073 =  -0.456166
BTFIT_C_IR_087 =  -1.507390
BTFIT_C_IR_097 =  -1.030600
BTFIT_C_IR_108 =  -3.296740
BTFIT_C_IR_120 =  -3.181090
BTFIT_C_IR_134 =  -2.645950


C1 = 1.19104273e-16
C2 = 0.0143877523

class _Calibrator(object):
    def __init__(self, hdr, channel_name):
        self.hdr = hdr
        self.channel_name = channel_name
        
    def __call__(self, image, calibrate=1):
        """Computes the radiances and reflectances/bt of a given channel.  The
        *calibrate* argument should be set to 0 for no calibration, 1 for
        default reflectances/bt calibration, and 2 for returning radiances. The
        default value is 1.
        """
        hdr = self.hdr

        channel_name = self.channel_name

        if calibrate == 0:
            return (image, 
                    "counts")

        channels = {"VIS006": 1,
                    "VIS008": 2,
                    "IR_016": 3,
                    "IR_039": 4,
                    "WV_062": 5,
                    "WV_073": 6,
                    "IR_087": 7,
                    "IR_097": 8,
                    "IR_108": 9,
                    "IR_120": 10,
                    "IR_134": 11,
                    "HRV": 12}

        cal_type = (hdr["Level 1_5 ImageProduction"]["PlannedChanProcessing"])
        chn_nb = channels[channel_name] - 1

        mask = (image == no_data_value)
        
        cslope = hdr["Level1_5ImageCalibration"][chn_nb]['Cal_Slope']
        coffset = hdr["Level1_5ImageCalibration"][chn_nb]['Cal_Offset']
        
        radiances = eval_np('image * cslope + coffset')
        radiances[radiances < 0] = 0
        
        if calibrate == 2:
            return (np.ma.MaskedArray(radiances, mask=mask),
                    "mW m-2 sr-1 (cm-1)-1")
            
        
        sat = hdr["SatelliteDefinition"]["SatelliteId"]
        if sat not in CALIB:
            raise CalibrationError("No calibration coefficients available for "
                                   + "this satellite (" + str(sat) + ")")

        if channel_name in ["HRV", "VIS006", "VIS008", "IR_016"]:
            solar_irradiance = CALIB[sat][channel_name]["F"]
            reflectance = eval_np('(radiances / solar_irradiance) * 100.')
            return (np.ma.MaskedArray(reflectance, mask=mask), 
                    "%")

        wavenumber = CALIB[sat][channel_name]["VC"]
        if cal_type[chn_nb] == 2:
            #computation based on effective radiance
            alpha = CALIB[sat][channel_name]["ALPHA"]
            beta = CALIB[sat][channel_name]["BETA"]
            
            cal_data = eval_np(('((C2 * 100. * wavenumber / ' 
                                'log(C1 * 1.0e6 * wavenumber ** 3 / ' 
                                '(1.0e-5 * radiances) + 1)) - beta) / alpha'))
            
        elif cal_type[chn_nb] == 1:
            #computation based on spectral radiance
            cal_data = eval_np(('C2 * 100. * wavenumber / '
                                'log(C1 * 1.0e6 * wavenumber ** 3 / '
                                '(1.0e-5 * radiances) + 1))'))
            
            coef_a = eval("BTFIT_A_" + channel_name)
            coef_b = eval("BTFIT_B_" + channel_name)
            coef_c = eval("BTFIT_C_" + channel_name)
            
            cal_data = eval_np(('cal_data ** 2 * coef_a + ' 
                                'cal_data * coef_b + coef_c'))

        else:
            raise RuntimeError("Something is seriously wrong in the metadata.")

        mask = mask | np.isnan(cal_data) | np.isinf(cal_data)
        cal_data = np.ma.MaskedArray(cal_data, mask=mask)
        return (cal_data, 
                "K")

def read_proheader(fp):
    """Read the msg header.
    """
    hdr = dict()

    # Satellite definition

    satdef = {}
    satdef["SatelliteId"] = rbin.read_uint2(fp.read(2))
    satdef["NominalLongitude"] = rbin.read_float4(fp.read(4))
    satdef["SatelliteStatus"] = ord(fp.read(1))

    hdr["SatelliteDefinition"] = satdef
    del satdef

    # Satellite operations

    satop = {}
    satop["LastManoeuvreFlag"] = ord(fp.read(1)) > 0
    satop["LastManoeuvreStartTime"] = rbin.read_cds_time(fp.read(6))
    satop["LastManoeuvreEndTime"] = rbin.read_cds_time(fp.read(6))
    satop["LastManoeuvreType"] =  ord(fp.read(1))
    satop["NextManoeuvreFlag"] = ord(fp.read(1)) > 0
    satop["NextManoeuvreStartTime"] = rbin.read_cds_time(fp.read(6))
    satop["NextManoeuvreEndTime"] = rbin.read_cds_time(fp.read(6))
    satop["NextManoeuvreType"] =  ord(fp.read(1))

    hdr["SatelliteOperations"] = satop
    del satop

    # Orbit

    orbit = {}
    orbit["PeriodStartTime"] = rbin.read_cds_time(fp.read(6))
    orbit["PeriodEndTime"] = rbin.read_cds_time(fp.read(6))
    orbitcoef = np.dtype(">u2, >u4, >u2, >u4,"
                         " (8,)>f8, (8,)>f8, (8,)>f8,"
                         " (8,)>f8, (8,)>f8, (8,)>f8")
    orbit["OrbitPolynomial"] = np.fromstring(fp.read(39600),
                                             dtype=orbitcoef,
                                             count=100)

    hdr["Orbit"] = orbit
    del orbit

    # Attitude

    attitude = {}
    attitude["PeriodStartTime"] = rbin.read_cds_time(fp.read(6))
    attitude["PeriodEndTime"] =  rbin.read_cds_time(fp.read(6))
    attitude["PrincipleAxisOffsetAngle"] = rbin.read_float8(fp.read(8))
    attitudecoef = np.dtype(">u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8, (8,)>f8")
    attitude["AttitudePolynomial"] = np.fromstring(fp.read(20400),
                                                   dtype=attitudecoef,
                                                   count=100)

    hdr["Attitude"] = attitude
    del attitude
    
    # SpinRateatRCStart
    
    hdr["SpinRateatRCStart"] = rbin.read_float8(fp.read(8))

    # UTCCorrelation

    utccor = {}
    
    utccor["PeriodStartTime"] = rbin.read_cds_time(fp.read(6))
    utccor["PeriodEndTime"] = rbin.read_cds_time(fp.read(6))
    utccor["OnBoardTimeStart"] = rbin.read_cuc_time(fp.read(7), 4, 3)
    utccor["VarOnBoardTimeStart"] = rbin.read_float8(fp.read(8))
    utccor["A1"] = rbin.read_float8(fp.read(8))
    utccor["VarA1"] = rbin.read_float8(fp.read(8))
    utccor["A2"] = rbin.read_float8(fp.read(8))
    utccor["VarA2"] = rbin.read_float8(fp.read(8))

    hdr["UTCCorrelation"] = utccor
    del utccor

    # PlannedAcquisitionTime

    pat = {}
    pat["TrueRepeatCycleStart"] = rbin.read_cds_expanded_time(fp.read(10))
    pat["PlannedForwardScanEnd"] = rbin.read_cds_expanded_time(fp.read(10))
    pat["PlannedRepeatCycleEnd"] = rbin.read_cds_expanded_time(fp.read(10))

    hdr["PlannedAcquisitionTime"] = pat

    # RadiometerStatus
    
    radiostatus = {}
    radiostatus["ChannelStatus"] = np.fromstring(fp.read(12), dtype=np.uint8)
    radiostatus["DetectorStatus"] = np.fromstring(fp.read(42), dtype=np.uint8)

    hdr["RadiometerStatus"] = radiostatus

    # RadiometerSettings

    radiosettings = {}
    radiosettings["MDUSamplingDelays"] = np.fromstring(fp.read(42 * 2), dtype=">u2")
    radiosettings["HRVFrameOffsets"] = {}
    radiosettings["HRVFrameOffsets"]["MDUNomHRVDelay1"] = rbin.read_uint2(fp.read(2))
    radiosettings["HRVFrameOffsets"]["MDUNomHRVDelay2"] = rbin.read_uint2(fp.read(2))
    radiosettings["HRVFrameOffsets"]["Spare"] = rbin.read_uint2(fp.read(2))
    radiosettings["HRVFrameOffsets"]["MDUNomHRVBreakline"] = rbin.read_uint2(fp.read(2))
    radiosettings["DHSSSynchSelection"] = ord(fp.read(1))
    radiosettings["MDUOutGain"] = np.fromstring(fp.read(42 * 2), dtype=">u2")
    radiosettings["MDUCourseGain"] = np.fromstring(fp.read(42), dtype=np.uint8)
    radiosettings["MDUFineGain"] = np.fromstring(fp.read(42 * 2), dtype=">u2")
    radiosettings["MDUNumericalOffset"] = np.fromstring(fp.read(42 * 2), dtype=">u2")
    radiosettings["PUGain"] = np.fromstring(fp.read(42 * 2), dtype=">u2")
    radiosettings["PUOffset"] = np.fromstring(fp.read(27 * 2), dtype=">u2")
    radiosettings["PUBias"] = np.fromstring(fp.read(15 * 2), dtype=">u2")
    radiosettings["OperationParameters"] = {}
    radiosettings["OperationParameters"]["L0_LineCounter"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["K1_RetraceLines"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["K2_PauseDeciseconds"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["K3_RetraceLines"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["K4_PauseDeciseconds"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["K5_RetraceLines"] = rbin.read_uint2(fp.read(2))
    radiosettings["OperationParameters"]["X_DeepSpaceWindowPosition"] = ord(fp.read(1))
    radiosettings["RefocusingLines"] = rbin.read_uint2(fp.read(2))
    radiosettings["RefocusingDirection"] = ord(fp.read(1))
    radiosettings["RefocusingPosition"] = rbin.read_uint2(fp.read(2))
    radiosettings["ScanRefPosFlag"] = ord(fp.read(1)) > 0
    radiosettings["ScanRefPosNumber"] = rbin.read_uint2(fp.read(2))
    radiosettings["ScanRefPosVal"] = rbin.read_float4(fp.read(4))
    radiosettings["ScanFirstLine"] = rbin.read_uint2(fp.read(2))
    radiosettings["ScanLastLine"] = rbin.read_uint2(fp.read(2))
    radiosettings["RetraceStartLine"] = rbin.read_uint2(fp.read(2))

    hdr["RadiometerSettings"] = radiosettings

    # RadiometerOperations

    radiooper = {}

    radiooper["LastGainChangeFlag"] = ord(fp.read(1)) > 0
    radiooper["LastGainChangeTime"] = rbin.read_cds_time(fp.read(6))
    radiooper["Decontamination"] = {}
    radiooper["Decontamination"]["DecontaminationNow"] = ord(fp.read(1)) > 0
    radiooper["Decontamination"]["DecontaminationStart"] = rbin.read_cds_time(fp.read(6))
    radiooper["Decontamination"]["DecontaminationEnd"] = rbin.read_cds_time(fp.read(6))


    radiooper["BBCalScheduled"] = ord(fp.read(1)) > 0
    radiooper["BBCalibrationType"] = ord(fp.read(1))
    radiooper["BBFirstLine"] = rbin.read_uint2(fp.read(2))
    radiooper["BBLastLine"] = rbin.read_uint2(fp.read(2))
    radiooper["ColdFocalPlaneOpTemp"] = rbin.read_uint2(fp.read(2))
    radiooper["WarmFocalPlaneOpTemp"] = rbin.read_uint2(fp.read(2))


    hdr["RadiometerOperations"] = radiooper

    ## CelestialEvents
    # CelestialBodiesPosition

    celbodies = {}
    celbodies["PeriodTimeStart"] = rbin.read_cds_time(fp.read(6))
    celbodies["PeriodTimeEnd"] = rbin.read_cds_time(fp.read(6))
    celbodies["RelatedOrbitFileTime"] = fp.read(15)
    celbodies["RelatedAttitudeFileTime"] = fp.read(15)
    earthmoonsuncoef = np.dtype(">u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8")
    celbodies["EarthEphemeris"] = np.fromstring(fp.read(14000),
                                             dtype=earthmoonsuncoef,
                                             count=100)
    celbodies["MoonEphemeris"] = np.fromstring(fp.read(14000),
                                             dtype=earthmoonsuncoef,
                                             count=100)
    celbodies["SunEphemeris"] = np.fromstring(fp.read(14000),
                                             dtype=earthmoonsuncoef,
                                             count=100)
    starcoef = np.dtype(">u2, >u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8")
    starcoefs = np.dtype([('starcoefs', starcoef, (20,))])

    celbodies["StarEphemeris"] = np.fromstring(fp.read(284000),
                                               dtype=starcoefs,
                                               count=100)

    hdr["CelestialBodiesPosition"] = celbodies

    # RelationToImage

    reltoim = {}
    reltoim["TypeofEclipse"] = ord(fp.read(1))
    reltoim["EclipseStartTime"] = rbin.read_cds_time(fp.read(6))
    reltoim["EclipseEndTime"] = rbin.read_cds_time(fp.read(6))
    reltoim["VisibleBodiesInImage"] = ord(fp.read(1))
    reltoim["BodiesClosetoFOV"] = ord(fp.read(1))
    reltoim["ImpactOnImageQuality"] = ord(fp.read(1))

    hdr["RelationToImage"] = reltoim

    ## ImageDescriptionRecord

    grid_origin = ["north west", "south west", "south east", "north east"]

    # ProjectionDescription

    projdes = {}
    projdes["TypeOfProjection"] = ord(fp.read(1))
    projdes["LongitudeOfSSP"] = rbin.read_float4(fp.read(4))

    hdr["ProjectionDescription"] = projdes

    # ReferenceGridVIS_IR

    refvisir = {}
    refvisir["NumberOfLines"] = rbin.read_int4(fp.read(4))
    refvisir["NumberOfColumns"] = rbin.read_int4(fp.read(4))
    refvisir["LineDirGridStep"] = rbin.read_float4(fp.read(4))
    refvisir["ColumnDirGridStep"] = rbin.read_float4(fp.read(4))
    refvisir["GridOrigin"] = grid_origin[ord(fp.read(1))]

    hdr["ReferenceGridVIS_IR"] = refvisir

    # ReferenceGridHRV

    refhrv = {}
    refhrv["NumberOfLines"] = rbin.read_int4(fp.read(4))
    refhrv["NumberOfColumns"] = rbin.read_int4(fp.read(4))
    refhrv["LineDirGridStep"] = rbin.read_float4(fp.read(4))
    refhrv["ColumnDirGridStep"] = rbin.read_float4(fp.read(4))
    refhrv["GridOrigin"] = grid_origin[ord(fp.read(1))]

    hdr["ReferenceGridHRV"] = refhrv

    # PlannedCoverageVIS_IR

    covvisir = {}
    covvisir["SouthernLinePlanned"] = rbin.read_int4(fp.read(4))
    covvisir["NorthernLinePlanned"] = rbin.read_int4(fp.read(4))
    covvisir["EasternColumnPlanned"] = rbin.read_int4(fp.read(4))
    covvisir["WesternColumnPlanned"] = rbin.read_int4(fp.read(4))

    hdr["PlannedCoverageVIS_IR"] = covvisir
    
    # PlannedCoverageHRV

    covhrv = {}
    
    covhrv["LowerSouthLinePlanned"] = rbin.read_int4(fp.read(4))
    covhrv["LowerNorthLinePlanned"] = rbin.read_int4(fp.read(4))
    covhrv["LowerEastColumnPlanned"] = rbin.read_int4(fp.read(4))
    covhrv["LowerWestColumnPlanned"] = rbin.read_int4(fp.read(4))
    covhrv["UpperSouthLinePlanned"] = rbin.read_int4(fp.read(4))
    covhrv["UpperNorthLinePlanned"] = rbin.read_int4(fp.read(4))
    covhrv["UpperEastColumnPlanned"] = rbin.read_int4(fp.read(4))
    covhrv["UpperWestColumnPlanned"] = rbin.read_int4(fp.read(4))

    hdr["PlannedCoverageHRV"] = covhrv

    # Level 1_5 ImageProduction

    image_proc_direction = ["North-South", "South-North"]
    pixel_gen_direction = ["East-West", "West-East"]
    
    l15prod = {}
    l15prod["ImageProcDirection"] = image_proc_direction[ord(fp.read(1))]
    l15prod["PixelGenDirection"] = pixel_gen_direction[ord(fp.read(1))]

    # 0: No processing, 1: Spectral radiance, 2: Effective radiance
    l15prod["PlannedChanProcessing"] = np.fromstring(fp.read(12),
                                                     dtype=np.uint8)

    hdr["Level 1_5 ImageProduction"] = l15prod


    ## RadiometricProcessing

    # RPSummary

    rpsummary = {}
    rpsummary["RadianceLinearization"] = np.fromstring(fp.read(12), dtype=np.bool)
    
    rpsummary["DetectorEqualization"] = np.fromstring(fp.read(12), dtype=np.bool)
    rpsummary["OnboardCalibrationResult"] = np.fromstring(fp.read(12), dtype=np.bool)
    rpsummary["MPEFCalFeedback"] = np.fromstring(fp.read(12), dtype=np.bool)
    rpsummary["MTFAdaptation"] = np.fromstring(fp.read(12), dtype=np.bool)
    rpsummary["StraylightCorrectionFlag"] = np.fromstring(fp.read(12), dtype=np.bool)

    hdr["RPSummary"] = rpsummary

    # Level1_5ImageCalibration

    caltype = np.dtype([('Cal_Slope', '>f8'), ('Cal_Offset', '>f8')])

    hdr["Level1_5ImageCalibration"] = np.fromstring(fp.read(192), dtype=caltype)


    # BlackBodyDataUsed

    bbdu = {}

    bbdu["BBObservationUTC"] = rbin.read_cds_expanded_time(fp.read(10))
    bbdu["BBRelatedData"] = {}
    bbdu["BBRelatedData"]["OnBoardBBTime"] = rbin.read_cuc_time(fp.read(7), 4, 3)
    bbdu["BBRelatedData"]["MDUOutGain"] = np.fromstring(fp.read(42 * 2),
                                                        dtype=">u2")
    bbdu["BBRelatedData"]["MDUCoarseGain"] = np.fromstring(fp.read(42),
                                                        dtype=np.uint8)
    bbdu["BBRelatedData"]["MDUFineGain"] = np.fromstring(fp.read(42 * 2),
                                                        dtype=">u2")
    bbdu["BBRelatedData"]["MDUNumericalOffset"] = np.fromstring(fp.read(42 * 2),
                                                        dtype=">u2")
    bbdu["BBRelatedData"]["PUGain"] = np.fromstring(fp.read(42 * 2),
                                                        dtype=">u2")
    bbdu["BBRelatedData"]["PUOffset"] = np.fromstring(fp.read(27 * 2),
                                                        dtype=">u2")
    bbdu["BBRelatedData"]["PUBias"] = np.fromstring(fp.read(15 * 2),
                                                        dtype=">u2")
    # 12 bits bitstrings... convert to uint16
    data = np.fromstring(fp.read(int(42 * 1.5)),
                         dtype=np.uint8)
    data = data.astype(np.uint16)
    data[::3] = data[::3]*256 + data[1::3] // 16
    data[1::3] = (data[1::3] & 0x0f)*16 + data[2::3]
    result = np.ravel(data.reshape(-1,3)[:,:2])
    bbdu["BBRelatedData"]["DCRValues"] = result
    bbdu["BBRelatedData"]["X_DeepSpaceWindowPosition"] = ord(fp.read(1))
    bbdu["BBRelatedData"]["ColdFPTemperature"] = {}
    bbdu["BBRelatedData"]["ColdFPTemperature"]["FCUNominalColdFocalPlaneTemp"] = rbin.read_uint2(fp.read(2)) / 100.
    bbdu["BBRelatedData"]["ColdFPTemperature"]["FCURedundantColdFocalPlaneTemp"] = rbin.read_uint2(fp.read(2)) / 100.
    bbdu["BBRelatedData"]["WarmFPTemperature"] = {}
    bbdu["BBRelatedData"]["WarmFPTemperature"]["FCUNominalWarmFocalPlaneVHROTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["WarmFPTemperature"]["FCURedundantWarmFocalPlaneVHROTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["ScanMirrorTemperature"] = {}
    bbdu["BBRelatedData"]["ScanMirrorTemperature"]["FCUNominalScanMirrorSensor1Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["ScanMirrorTemperature"]["FCURedundantScanMirrorSensor1Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["ScanMirrorTemperature"]["FCUNominalScanMirrorSensor2Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["ScanMirrorTemperature"]["FCURedundantScanMirrorSensor2Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["M1M2M3Temperature"] = {}
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCUNominalM1MirrorSensor1Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCURedundantM1MirrorSensor1Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCUNominalM1MirrorSensor2Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCURedundantM1MirrorSensor2Temp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCUNominalM23AssemblySensor1Temp"] = ord(fp.read(1)) / 4. + 265
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCURedundantM23AssemblySensor1Temp"] = ord(fp.read(1)) / 4. + 265
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCUNominalM23AssemblySensor2Temp"] = ord(fp.read(1)) / 4. + 265
    bbdu["BBRelatedData"]["M1M2M3Temperature"]["FCURedundantM23AssemblySensor2Temp"] = ord(fp.read(1)) / 4. + 265
    bbdu["BBRelatedData"]["BaffleTemperature"] = {}
    bbdu["BBRelatedData"]["BaffleTemperature"]["FCUNominalM1BaffleTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["BaffleTemperature"]["FCURedundantM1BaffleTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["BlackBodyTemperature"] = {}
    bbdu["BBRelatedData"]["BlackBodyTemperature"]["FCUNominalBlackBodySensorTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["BlackBodyTemperature"]["FCURedundantBlackBodySensorTemp"] = rbin.read_uint2(fp.read(2)) / 100. + 250
    bbdu["BBRelatedData"]["FCUMode"] = {}
    bbdu["BBRelatedData"]["FCUMode"]["FCUNominalSMMStatus"] = rbin.read_uint2(fp.read(2))
    bbdu["BBRelatedData"]["FCUMode"]["FCURedundantSMMStatus"] = rbin.read_uint2(fp.read(2))
    extracted_data_type = np.dtype([('NumberOfPixelsUsed', '>u4'),
                                    ('MeanCount', '>f4'),
                                    ('RMS', '>f4'),
                                    ('MaxCount', '>u2'),
                                    ('MinCount', '>u2'),
                                    ('BB_Processing_Slope', '>f8'),
                                    ('BB_Processing_Offset', '>f8')])
    
    bbdu["BBRelatedData"]["ExtractedBBData"] = np.fromstring(fp.read(32 * 12),
                                                             dtype=extracted_data_type)
    impf_cal_type = np.dtype([("ImageQualityFlag", "u1"),
                              ("ReferenceDataFlag", "u1"),
                              ("AbsCalMethod", "u1"),
                              ("Pad1", "u1"),
                              ("AbsCalWeightVic", ">f4"),
                              ("AbsCalWeightXsat", ">f4"),
                              ("AbsCalCoeff", ">f4"),
                              ("AbsCalError", ">f4"),
                              ("CalMonBias", ">f4"),
                              ("CalMonRms", ">f4"),
                              ("OffsetCount", ">f4")])

    
    bbdu["MPEFCalFeedback"] = np.fromstring(fp.read(32 * 12),
                                            dtype=impf_cal_type)
    
    bbdu["RadTransform"] = np.fromstring(fp.read(42 * 64 * 4),
                                            dtype=">f4").reshape((42,64))
    bbdu["RadProcMTFAdaptation"] = {}
    
    bbdu["RadProcMTFAdaptation"]["VIS_IRMTFCorrectionE_W"] = np.fromstring(fp.read(33 * 16 * 4),
                                                                           dtype=">f4").reshape((33, 16))
    bbdu["RadProcMTFAdaptation"]["VIS_IRMTFCorrectionN_S"] = np.fromstring(fp.read(33 * 16 * 4),
                                                                           dtype=">f4").reshape((33, 16))
    bbdu["RadProcMTFAdaptation"]["HRVMTFCorrectionE_W"] = np.fromstring(fp.read(9 * 16 * 4),
                                                                        dtype=">f4").reshape((9, 16))
    bbdu["RadProcMTFAdaptation"]["HRVMTFCorrectionN_S"] = np.fromstring(fp.read(9 * 16 * 4),
                                                                        dtype=">f4").reshape((9, 16))
    bbdu["RadProcMTFAdaptation"]["StraylightCorrection"] = np.fromstring(fp.read(12 * 8 * 8 * 4),
                                                                        dtype=">f4").reshape((12, 8, 8))

    hdr["BlackBodyDataUsed"] = bbdu

    # GeometricProcessing

    geoproc = {}
    geoproc["OptAxisDistances"] = {}
    geoproc["OptAxisDistances"]["E-WFocalPlane"] = np.fromstring(fp.read(42 * 4),
                                                                 dtype=">f4")
    geoproc["OptAxisDistances"]["N-SFocalPlane"] = np.fromstring(fp.read(42 * 4),
                                                                 dtype=">f4")

    geoproc["EarthModel"] = {}
    geoproc["EarthModel"]["TypeOfEarthModel"] = ord(fp.read(1))
    geoproc["EarthModel"]["EquatorialRadius"] = rbin.read_float8(fp.read(8))
    geoproc["EarthModel"]["NorthPolarRadius"] = rbin.read_float8(fp.read(8))
    geoproc["EarthModel"]["SouthPolarRadius"] = rbin.read_float8(fp.read(8))
    geoproc["AtmosphericModel"] = np.fromstring(fp.read(12 * 360 * 4),
                                                dtype=">f4").reshape((12, 360))
    geoproc["ResamplingFunctions"] = np.fromstring(fp.read(12),
                                                   dtype=np.uint8)

    hdr["GeometricProcessing"] = geoproc

    return hdr

def read_epiheader(fp):
    """Read the msg header.
    """
    ftr = dict()
    ftr["15TRAILERVersion"] = ord(fp.read(1))
    ftr["SateliteID"] = rbin.read_uint2(fp.read(2))
    ftr["NominalImageScanning"] = ord(fp.read(1)) > 0
    ftr["ReducedScan"] = ord(fp.read(1)) > 0
    ftr["ForwardScanStart"] = rbin.read_cds_time(fp.read(6))
    ftr["ForwardScanEnd"] = rbin.read_cds_time(fp.read(6))
    ftr["NominalBehaviour"] = ord(fp.read(1)) > 0
    ftr["RadScanIrregularity"] = ord(fp.read(1)) > 0
    ftr["RadStoppage"] = ord(fp.read(1)) > 0
    ftr["RepeatCycleNotCompleted"] = ord(fp.read(1)) > 0
    ftr["GainChangeTookPlace"] = ord(fp.read(1)) > 0
    ftr["DecontaminationTookPlace"] = ord(fp.read(1)) > 0
    ftr["NoBBCalibrationAchieved"] = ord(fp.read(1)) > 0
    ftr["IncorrectTemperature"] = ord(fp.read(1)) > 0
    ftr["InvalidBBData"] = ord(fp.read(1)) > 0
    ftr["InvalidAuxOrHKTMData"] = ord(fp.read(1)) > 0
    ftr["RefocusingMechanismActuated"] = ord(fp.read(1)) > 0
    ftr["MirrorBackToReferencePos"] = ord(fp.read(1)) > 0
    ftr["PlannedNumberOfL10Lines"] = np.fromstring(fp.read(12 * 4),
                                                   dtype=">u4")
    ftr["NumberOfMissingL10Lines"] = np.fromstring(fp.read(12 * 4),
                                                   dtype=">u4")
    ftr["NumberOfCorruptedL10Lines"] = np.fromstring(fp.read(12 * 4),
                                                   dtype=">u4")
    ftr["NumberOfReplacedL10Lines"] = np.fromstring(fp.read(12 * 4),
                                                   dtype=">u4")
    validitytype = np.dtype([('NominalImage', '>u1'),
                             ('NonNominalBecauseIncomplete', '>u1'),
                             ('NonNominalRadiometricQuality', '>u1'),
                             ('NonNominalGeometricQuality', '>u1'),
                             ('NonNominalTimeliness', '>u1'),
                             ('IncompleteL15', '>u1')])
    ftr["L15ImageValidity"] = np.fromstring(fp.read(12 * 6),
                                            dtype=validitytype)

    ftr["SouthernLineActual"] = rbin.read_int4(fp.read(4))
    ftr["NorthernLineActual"] = rbin.read_int4(fp.read(4))
    ftr["EasternColumnActual"] = rbin.read_int4(fp.read(4))
    ftr["WesternColumnActual"] = rbin.read_int4(fp.read(4))
    ftr["LowerSouthLineActual"] = rbin.read_int4(fp.read(4))
    ftr["LowerNorthLineActual"] = rbin.read_int4(fp.read(4))
    ftr["LowerEastColumnActual"] = rbin.read_int4(fp.read(4))
    ftr["LowerWestColumnActual"] = rbin.read_int4(fp.read(4))
    ftr["UpperSouthLineActual"] = rbin.read_int4(fp.read(4))
    ftr["UpperNorthLineActual"] = rbin.read_int4(fp.read(4))
    ftr["UpperEastColumnActual"] = rbin.read_int4(fp.read(4))
    ftr["UpperWestColumnActual"] = rbin.read_int4(fp.read(4))

    return ftr

def read_metadata(prologue, image_files, epilogue):
    """ Selected items from the Meteosat-9 prolog file.
    """
    segment_size = 464 # number of lines in a segment

    fp = StringIO(prologue.data)
    hdr = read_proheader(fp)

    fp = StringIO(epilogue.data)
    ftr = read_epiheader(fp)
    
    im = _xrit.read_imagedata(image_files[0])

    md = Metadata()
    md.calibrate = _Calibrator(hdr, im.product_name)

    md.sublon = hdr["ProjectionDescription"]["LongitudeOfSSP"]
    md.product_name = im.product_id
    md.channel = im.product_name
    if md.channel == "HRV":
        md.image_size = np.array((hdr["ReferenceGridHRV"]["NumberOfLines"],
                                  hdr["ReferenceGridHRV"]["NumberOfColumns"]))
    else:
        md.image_size = np.array((hdr["ReferenceGridVIS_IR"]["NumberOfLines"],
                                  hdr["ReferenceGridVIS_IR"]["NumberOfColumns"]))
        
    md.satname = im.platform.lower()
    md.product_type = 'full disc'
    md.region_name = 'full disc'
    if md.channel == "HRV":
        md.first_pixel = hdr["ReferenceGridHRV"]["GridOrigin"]
        ns_, ew_ = md.first_pixel.split()
        md.boundaries = np.array([[
            ftr["LowerSouthLineActual"],
            ftr["LowerNorthLineActual"],
            ftr["LowerEastColumnActual"],
            ftr["LowerWestColumnActual"]],
           [ftr["UpperSouthLineActual"],
            ftr["UpperNorthLineActual"],
            ftr["UpperEastColumnActual"],
            ftr["UpperWestColumnActual"]]])

        im_loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
        md.coff = (ftr["Lower"+ew_.capitalize()+"ColumnActual"]
                   + im.navigation.coff - 1)
        md.loff = (ftr["Lower"+ns_.capitalize()+"LineActual"]
                   + im_loff - 1)

    else:
        md.first_pixel = hdr["ReferenceGridVIS_IR"]["GridOrigin"]
        ns_, ew_ = md.first_pixel.split()
        md.boundaries = np.array([[
            ftr["SouthernLineActual"],
            ftr["NorthernLineActual"],
            ftr["EasternColumnActual"],
            ftr["WesternColumnActual"]]])

        im_loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
        md.coff = (ftr[ew_.capitalize()+"ernColumnActual"]
                   + im.navigation.coff - 1)
        md.loff = (ftr[ns_.capitalize()+"ernLineActual"]
                   + im_loff - 1)

    md.data_type = im.structure.nb
    md.no_data_value = no_data_value
    md.line_offset = 0
    md.time_stamp = im.time_stamp
    md.production_time = im.production_time
    md.calibration_unit = 'counts'

    return md

if __name__ == '__main__':
    p = _xrit.read_prologue(sys.argv[1])
    e = _xrit.read_epilogue(sys.argv[-1])
    print read_metadata(p, sys.argv[2:-1], e)



#
# $Id$
#
from datetime import datetime
import numpy

def mslice(mda):
    _mda = Metadata()
    for key, val in mda.__dict__.items():
        if (not key.startswith('_') and 
            not callable(val) and
            key not in mda.ignore_attributes):
            setattr(_mda, key, val)
    return _mda

class Metadata(object):
    token = ':'
    ignore_attributes = ()
    dont_eval = ('satnumber',)

    
    def read(self, file_name):
        """Read until empty line, 'EOH' or 'EOF'.
        """
        fpi = open(file_name)
        try:
            for line in fpi:
                line = line.strip()
                if not line or line == 'EOH':
                    # end of meta-data
                    break
                line = line.split('#')[0].strip()
                if not line:
                    # just a comment
                    continue
                key, val = [s.strip() for s in line.split(self.token, 1)]
                if key not in self.dont_eval:
                    try:
                        val = eval(val)
                    except:
                        pass
                if key:
                    setattr(self, key, val)
        finally:
            fpi.close()
        return self

    def save(self, file_name):
        fpo = open(file_name, 'w')
        fpo.write(str(self) + '\n')
        fpo.close()

    def __str__(self):
        keys = sorted(self.__dict__.keys())
        strn = ''
        for key in keys:
            val = getattr(self, key)
            if (not key.startswith('_') and 
                not callable(val) and
                key not in self.ignore_attributes):
                val = _nice2cmp(val)
                strn += key + self.token + ' ' + str(val) + '\n'
        return strn[:-1]

def _nice2cmp(val):
    # ... and nice to print
    if isinstance(val, numpy.ndarray):
        val = val.tolist()
    elif isinstance(val, datetime):
        val = str(val)
    elif isinstance(val, float):
        val = str(val)
    elif isinstance(val, dict):
        sdc = {}
        for _key, _val in val.items():
            if isinstance(_val, numpy.ndarray):
                _val = _val.tolist()
            sdc[_key] = _val
        val = sdc
    return val

if __name__ == '__main__':
    import sys
    print Metadata().read(sys.argv[1])



from osgeo import gdal, osr

import logging
logger = logging.getLogger('mipp')

def tiff2areadef(projection, geotransform, shape):
    # Rewamp projection
    import pyresample

    srs = osr.SpatialReference()
    srs.ImportFromWkt(projection)
    proj4 = srs.ExportToProj4()
    proj4_dict = {}
    for i in proj4.replace('+', '').split():
        try:
            key, val = [v.strip() for v in i.split('=')]
        except ValueError:
            continue        
        proj4_dict[key] = val

    area_extent = [geotransform[0],
                   geotransform[3] + geotransform[5]*shape[0],
                   geotransform[0] + geotransform[1]*shape[1],
                   geotransform[3]]
    aid = proj4_dict['proj']
    if aid.lower() == 'utm':
        aid += proj4_dict['zone']
    # give it some kind of ID
    aname = aid + '_' + str(int(sum(geotransform)/1000.))

    return pyresample.utils.get_area_def(aname, aname, aid,
                                         proj4_dict,
                                         shape[1], shape[0],
                                         area_extent)

def read_geotiff(filename):
    dst = gdal.Open(filename)

    #
    # Dataset information
    #
    geotransform = dst.GetGeoTransform()
    projection = dst.GetProjection()
    metadata = dst.GetMetadata()

    logger.debug('description: %s'%dst.GetDescription())
    logger.debug('driver: %s / %s'%(dst.GetDriver().ShortName,
                                    dst.GetDriver().LongName))
    logger.debug('size: %d x %d x %d'%(dst.RasterXSize, dst.RasterYSize,
                                       dst.RasterCount))
    logger.debug('geo transform: %s'%str(geotransform))
    logger.debug('origin: %.3f, %.3f'%(geotransform[0], geotransform[3]))
    logger.debug('pixel size: %.3f, %.3f'%(geotransform[1], geotransform[5]))
    logger.debug('projection: %s'%projection)
    logger.debug('metadata: %s', metadata)

    #
    # Fetching raster data
    #
    band = dst.GetRasterBand(1)
    logger.info('Band(1) type: %s, size %d x %d'%(
            gdal.GetDataTypeName(band.DataType),
            dst.RasterXSize, dst.RasterYSize))
    shape = (dst.RasterYSize, dst.RasterXSize)
    if band.GetOverviewCount() > 0:
        logger.debug('overview count: %d'%band.GetOverviewCount())
    if not band.GetRasterColorTable() is None:
        logger.debug('colortable size: %d'%
                     band.GetRasterColorTable().GetCount())

    data = band.ReadAsArray(0, 0, shape[1], shape[0])
    logger.info('fetched array: %s %s %s [%d -> %.2f -> %d]'%
                (type(data), str(data.shape), data.dtype,
                 data.min(), data.mean(), data.max()))

    params = dict((('geotransform', geotransform),
                   ('projection', projection),
                   ('metadata', metadata)))

    return params, data



# Copyright (c) 2009-2012.

# DMI,
# Lyngbyvej 100
# DK-2100 Copenhagen
# Denmark

# Author(s):

#   Lars Orum Rasmussen <loerum@gmail.com>  

# This file is part of mipp.

# mipp is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# mipp is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with mipp.  If not, see <http://www.gnu.org/licenses/>.

"""Setup file for mipp.
"""
import os
from setuptools import setup, Extension

setup(name = 'mipp',
      description='Meteorological ingest processing package',
      author='Lars Orum Rasmussen',
      author_email='loerum@hmail.com',
      classifiers=["Development Status :: 5 - Production/Stable",
                   "Intended Audience :: Science/Research",
                   "License :: OSI Approved :: GNU General Public License v3 " +
                   "or later (GPLv3+)",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Scientific/Engineering"],
      url="https://github.com/loerum/mipp",
      version = '0.9.1',
      packages = ['mipp', 'mipp.xrit', 'mipp.xsar'],
      zip_safe = False,
      )



#
# $Id$ 
#

"""This module will read satellit data files in SGS (Support Ground Segments) format (eg. GOES, MTSAT).
Format described in:
'MSG Ground Segment LRIT/HRIT Mission Specific Implementation'; EUM/MSG/SPE/057; Issue 6; 21 June 2006
"""

import sys
import numpy

from mipp.xrit import _xrit
from mipp.xrit import Metadata
from mipp.xrit import bin_reader as rbin

no_data_value = 0

__all__ = ['read_metadata']

def _read_sgs_common_header(fp):
    hdr = dict()
    hdr['CommonHeaderVersion'] = rbin.read_uint1(fp.read(1))
    fp.read(3)
    hdr['NominalSGSProductTime'] = rbin.read_cds_time(fp.read(6))
    hdr['SGSProductQuality'] = rbin.read_uint1(fp.read(1))
    hdr['SGSProductCompleteness'] = rbin.read_uint1(fp.read(1))
    hdr['SGSProductTimeliness'] = rbin.read_uint1(fp.read(1))
    hdr['SGSProcessingInstanceId'] = rbin.read_uint1(fp.read(1))
    hdr['BaseAlgorithmVersion'] = fp.read(16).strip()
    hdr['ProductAlgorithmVersion'] = fp.read(16).strip()
    return hdr
    
def _read_sgs_product_header(fp):
    hdr = dict()
    hdr['ImageProductHeaderVersion'] = rbin.read_uint1(fp.read(1))
    fp.read(3)
    hdr['ImageProductHeaderLength'] = rbin.read_uint4(fp.read(4))
    hdr['ImageProductVersion'] = rbin.read_uint1(fp.read(1))
    #hdr['ImageProductHeaderData'] = fp.read()
    return hdr

class _Calibrator(object):
    def __init__(self, hdr):
        self.hdr = hdr

        dd = []
        for k in sorted(hdr.keys()):
            if isinstance(k, int):
                v = hdr[k]
                dd.append([float(k), v])
        self.calibration_table = numpy.array(dd, dtype=numpy.float32)

    def __call__(self, image, calibrate=1):
        cal = self.calibration_table

        if type(cal) != numpy.ndarray:
            cal = numpy.array(cal)

        if cal.shape == (256, 2):
            cal = cal[:,1] # nasty !!!
            cal[int(no_data_value)] = no_data_value
            image = cal[image] # this does not work on masked arrays !!!
        elif cal.shape == (2, 2):
            scale = (cal[1][1] - cal[0][1])/(cal[1][0] - cal[0][0])
            offset = cal[0][1] - cal[0][0]*scale
            image = numpy.select([image == no_data_value*scale], [no_data_value], default=offset + image*scale)
        else:
            raise mipp.DecodeError("Could not recognize the shape %s of the calibration table"%str(cal.shape))

        return (image,
                self.hdr['_UNIT'])

def read_metadata(prologue, image_files):
    """ Selected items from the GOES image data files (not much information in prologue).
    """
    im = _xrit.read_imagedata(image_files[0])
    hdr = im.data_function.data_definition
    md = Metadata()
    md.calibrate = _Calibrator(hdr)
    md.satname = im.platform.lower()
    md.product_type = 'full disc'
    md.region_name = 'full disc'
    md.product_name = prologue.product_id
    md.channel = prologue.product_name[:4]
    ssp = float(im.product_name[5:-1].replace('_','.'))
    if im.product_name[-1].lower() == 'w':            
        ssp *= -1
    md.sublon = ssp
    md.first_pixel = 'north west'
    md.data_type = im.structure.nb
    nseg = im.segment.planned_end_seg_no - im.segment.planned_start_seg_no + 1
    md.image_size = (im.structure.nc, im.structure.nl*nseg) # !!!
    md.line_offset = 0
    md.time_stamp = im.time_stamp
    md.production_time = im.production_time
    md.calibration_unit = 'counts'

    # Calibration table
    dd = []
    for k in sorted(hdr.keys()):
        if isinstance(k, int):
            v = hdr[k]
            dd.append([float(k), v])

    md.calibration_table = dict((('name', im.data_function.data_definition['_NAME']),
                                 ('unit', im.data_function.data_definition['_UNIT']),
                                 ('table', numpy.array(dd, dtype=numpy.float32))))

    md.no_data_value = no_data_value

    segment_size = im.structure.nl
    md.loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
    md.coff = im.navigation.coff

    return md

def read_prologue_headers(fp):
    hdr = _read_sgs_common_header(fp)
    hdr.update(_read_sgs_product_header(fp))
    return hdr
    
if __name__ == '__main__':
    print read_metadata(_xrit.read_prologue(sys.argv[1]), sys.argv[2:])



#
#
#
import copy
import numpy as np
from datetime import datetime
from lxml import etree
from osgeo import osr

import mipp
from mipp import geotiff
from mipp.xsar import Metadata

import logging
logger = logging.getLogger('mipp')

no_data_value = 0

def _tiff2areadef(projection, geotransform, shape):
    # revamp projection
    import pyresample

    srs = osr.SpatialReference()
    srs.ImportFromWkt(projection)
    proj4 = srs.ExportToProj4()
    proj4_dict = {}
    for i in proj4.replace('+', '').split():
        try:
            key, val = [val.strip() for val in i.split('=')]
        except ValueError:
            continue
        
        proj4_dict[key] = val
    area_extent = [geotransform[0],
                   geotransform[3] + geotransform[5]*shape[0],
                   geotransform[0] + geotransform[1]*shape[1],
                   geotransform[3]]
    aid = proj4_dict['proj']
    if aid.lower() == 'utm':
        aid += proj4_dict['zone']
    # give it some kind of ID
    aname = aid + '_' + str(int(sum(geotransform)/1000.))

    return pyresample.utils.get_area_def(aname, aname, aid,
                                         proj4_dict,
                                         shape[1], shape[0],
                                         area_extent)

class _Calibrator(object):
    def __init__(self, mda):
        self.factor = mda.calibration_factor
        self.unit = mda.calibration_unit.lower()
        self.error = None
        if mda.calibrated != 'CALIBRATED':
            self.error = "Data is not calibrated"
        if mda.beamid != mda.calibration_beamid:
            self.error = \
                "BeamID for image data and calibration factor don't match"
        if mda.calibration_unit.lower() != 'radar-brightness':
            self.error = "Unknown calibration unit '%s'" % (
                self.calibration_unit.lower())

    def __call__(self, image, calibrate=1):
        if calibrate == 0:
            return (image,
                    'counts')
        if self.error:
            raise mipp.CalibrationError, self.error
        return (image*image*self.factor,
                self.unit)

def read_metadata(xmlbuffer):

    # Speciel decoders
    def dec_isoformat(rts):
        return datetime.strptime(rts, "%Y-%m-%dT%H:%M:%S.%fZ")
    def dec_orbit_number(rts):
        return int(rts[:5])    
    def dec_satellite_name(rts):
        return rts.replace('-', '')
    def dec_calibration_unit(rts):
        _trans = {'radar brightness': 'nrcs'}
        rts = rts.replace(' ', '-').lower()
        return rts

    attributes = {
        'product_level':  ('generalHeader/itemName', str),
        'satellite_name': ('productInfo/missionInfo/mission', dec_satellite_name),
        'orbit_number': ('productInfo/missionInfo/absOrbit', dec_orbit_number),
        'sensor_type': ('productInfo/acquisitionInfo/sensor', str),
        'beam_mode': ('productInfo/acquisitionInfo/imagingMode', str),
        'polarisation': ('productInfo/acquisitionInfo/polarisationList/polLayer', str),
        'beamid': ('productInfo/acquisitionInfo/elevationBeamConfiguration', str),
        'calibrated': ('productInfo/productVariantInfo/radiometricCorrection', str),
        'calibration_factor': ('calibration/calibrationConstant/calFactor', float),
        'calibration_beamid': ('calibration/calibrationConstant/beamID', str),
        'calibration_unit': ('productInfo/imageDataInfo/pixelValueID', dec_calibration_unit),
        'image_data_path': ('productComponents/imageData/file/location/path', str),
        'image_data_filename': ('productComponents/imageData/file/location/filename', str),
        'time_start': ('productInfo/sceneInfo/start/timeUTC', dec_isoformat),        
        'center_coor_lat': ('productInfo/sceneInfo/sceneCenterCoord/lat', float),
        'center_coor_lon': ('productInfo/sceneInfo/sceneCenterCoord/lon', float)
        }

    check_attributes = {'product_level': 'level 1b product',
                        'satellite_name': 'tsx',
                        'sensor_type': 'sar'}

    tree = etree.fromstring(xmlbuffer)
    
    # Check satellite, sensor and product level
    for key, val in check_attributes.items():
        try:
            path = attributes[key][0]
            attr = tree.xpath(path)[0].text.lower()
            if not attr.startswith(val):
                raise mipp.ReaderError("This does not look like a TSX SAR " +
                                       "Level 1B Product, %s is '%s' expected '%s'" %
                                       (key, attr, val))
        except IndexError:
            raise mipp.ReaderError("This does not look like a TSX SAR " +
                                   "Level 1B Product, could not find attribute '%s' (%s)" %
                                   (key, path))

    mda = Metadata()
    for key, val in attributes.items():
        setattr(mda, key, val[1](tree.xpath(val[0])[0].text))
    mda.image_filename = (mda.image_data_path + '/' + mda.image_data_filename)
    delattr(mda, 'image_data_path')
    delattr(mda, 'image_data_filename')
    return mda

def read_image(mda, filename=None, mask=True, calibrate=1):
    mda = copy.copy(mda)
    mda.calibrate = _Calibrator(mda)
    mda.calibration_unit = 'counts'
    mda.is_calibrated = False
    del mda.calibrated
    mda.product_name = (mda.time_start.strftime("%Y%m%d_%H%M%S") + '_' + 
                        mda.satellite_name + '_' + mda.sensor_type + '_' + 
                        mda.beam_mode + '_' + mda.polarisation)
    logger.info('Product name: %s'% mda.product_name)
    
    if not filename:
        filename = mda.image_filename

    params, data = geotiff.read_geotiff(filename)
    area_def = geotiff.tiff2areadef(params['projection'],
                                    params['geotransform'],
                                    data.shape)

    mda.proj4_params = area_def.proj4_string.replace('+', '')
    mda.area_extent = area_def.area_extent
    mda.tiff_params = params

    mda.no_data_value = no_data_value

    if calibrate:
        data, mda.calibration_unit = mda.calibrate(data, calibrate)
        mda.is_calibrated = True
        logger.info('calibrated: %s %s [%.2f -> %.2f -> %.2f] %s'%
                    (str(data.shape), data.dtype, data.min(),
                     data.mean(), data.max(), mda.calibration_unit))

    if mask:
        mask = (data == no_data_value)
        data = np.ma.array(data, mask=mask, copy=False)

    return mda, data



import os
import logging as log

class NullHandler(log.Handler):
    """Empty handler.
    """
    def emit(self, record):
        """Record a message.
        """
        pass
    
def debug_on():
    """Turn debugging logging on.
    """
    logging_on(log.DEBUG)

_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
_is_logging_on = False
def logging_on(level=None):
    """Turn logging on.
    """
    global _is_logging_on

    if level == None:
        if os.environ.get("DEBUG", ''):
            level = log.DEBUG
        else:
            level = log.INFO

    if not _is_logging_on:
        console = log.StreamHandler()
        console.setFormatter(log.Formatter(_format, '%Y-%m-%d %H:%M:%S'))
        console.setLevel(level)
        log.getLogger('').addHandler(console)
        _is_logging_on = True

    logger = log.getLogger('')
    logger.setLevel(level)
    for handler in logger.handlers:
        handler.setLevel(level)

def logging_off():
    """Turn logging off.
    """
    global _is_logging_on
    logger = log.getLogger('')
    for handler in logger.handlers:
        handler.close()
        logger.removeHandler(handler)
    logger.handlers = [NullHandler()]
    _is_logging_on = False
    
def get_logger(name):
    """Return logger with null handle
    """
    
    logger = log.getLogger(name)
    if not logger.handlers:
        logger.addHandler(NullHandler())
    return logger
